#!/usr/bin/env python
# -*- coding: utf-8 -*-

# main.py
"""
The main file DVH Analytics
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

import wx
from datetime import datetime
import webbrowser
from pubsub import pub
from dvha.db import sql_columns
from dvha.db.sql_to_python import QuerySQL
from dvha.db.sql_connector import echo_sql_db, initialize_db
from dvha.dialogs.main import query_dlg, UserSettings, About, PythonLibraries
from dvha.dialogs.database import SQLSettingsDialog
from dvha.dialogs.export import ExportCSVDialog, ExportFigure
from dvha.dialogs.protocols import ProtocolsEditor
from dvha.models.import_dicom import ImportDicomFrame
from dvha.models.database_editor import DatabaseEditorFrame
from dvha.models.data_table import DataTable
from dvha.models.plot import PlotStatDVH
from dvha.models.dvh import DVH
from dvha.models.endpoint import EndpointFrame
from dvha.models.queried_data import QueriedDataFrame
from dvha.models.rad_bio import RadBioFrame
from dvha.models.time_series import TimeSeriesFrame
from dvha.models.correlation import CorrelationFrame
from dvha.models.machine_learning import MachineLearningModelViewer
from dvha.models.regression import RegressionFrame, LoadMultiVarModelFrame
from dvha.models.control_chart import ControlChartFrame
from dvha.models.roi_map import ROIMapFrame
from dvha.models.stats_data_editor import StatsDataEditor
from dvha.options import Options, DefaultOptions
from dvha.paths import LOGO_PATH, DATA_DIR, ICONS, MODELS_DIR
from dvha.tools.errors import MemoryErrorDialog, PlottingMemoryError
from dvha.tools.roi_name_manager import DatabaseROIs
from dvha.tools.stats import StatsData, sync_variables_in_stats_data_objects
from dvha.tools.utilities import get_study_instance_uids, scale_bitmap, is_windows, is_linux, is_mac, get_window_size, \
    save_object_to_file, load_object_from_file, set_msw_background_color, initialize_directories, \
    set_frame_icon
from dvha.db.sql_columns import all_columns as sql_column_info


class DVHAMainFrame(wx.Frame):
    def __init__(self, *args, **kwds):
        kwds["style"] = kwds.get("style", 0) | wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)

        self.layout_set = False

        self.sizer_dvhs = wx.BoxSizer(wx.VERTICAL)

        set_msw_background_color(self)  # If windows, change the background color

        self.options = Options()

        # Initial DVH object and data
        self.save_data = {}
        self.group_data = {1: {'dvh': None,
                               'data': {key: None for key in ['Plans', 'Beams', 'Rxs']},
                               'stats_data': None},
                           2: {'dvh': None,
                               'data': {key: None for key in ['Plans', 'Beams', 'Rxs']},
                               'stats_data': None}}

        self.toolbar_keys = ['Open', 'Close', 'Save', 'Export', 'Image', 'Import', 'Database', 'ROI Map', 'Settings']
        self.toolbar_ids = {key: i + 1000 for i, key in enumerate(self.toolbar_keys)}

        # sql_columns.py contains dictionaries of all queryable variables along with their
        # SQL columns and tables. Numerical categories include their units as well.
        self.categorical_columns = sql_columns.categorical
        self.numerical_columns = sql_columns.numerical

        # Keep track of currently selected row in the query tables
        self.selected_index_categorical = None
        self.selected_index_numerical = None

        # Load ROI Map now and pass to other objects for continuity
        # TODO: Need a method to address multiple users editing roi_map at the same time
        self.roi_map = DatabaseROIs()

        self.query_filters = None
        self.reset_query_filters()

        self.__add_menubar()
        self.__add_tool_bar()
        self.__add_layout_objects()
        self.__bind_layout_objects()

        columns = {'categorical': ['category_1', 'category_2', 'Filter Type'],
                   'numerical': ['category', 'min', 'max', 'Filter Type']}
        self.data_table_categorical = DataTable(self.table_categorical, columns=columns['categorical'])
        self.data_table_numerical = DataTable(self.table_numerical, columns=columns['numerical'])

        self.__set_properties()
        self.__set_tooltips()
        self.__add_notebook_frames()
        self.__do_layout()

        self.disable_query_buttons('categorical')
        self.disable_query_buttons('numerical')
        self.button_query_execute.Disable()
        self.__disable_notebook_tabs()

        self.Bind(wx.EVT_CLOSE, self.on_quit)
        self.tool_bar_windows = {key: None for key in ['import', 'database', 'roi_map']}
        self.loaded_mvr_ml_frames = []
        self.user_settings = None
        self.export_figure = None

        wx.CallAfter(self.__catch_failed_sql_connection_on_app_launch)

        self.__do_subscribe()

    def __add_tool_bar(self):
        self.frame_toolbar = wx.ToolBar(self, -1, style=wx.TB_HORIZONTAL | wx.TB_TEXT)
        self.SetToolBar(self.frame_toolbar)

        description = {'Open': "Open previously queried data",
                       'Close': "Clear queried data",
                       'Save': "Save queried data",
                       # 'Print': "Print a report",
                       'Export': "Export data to CSV",
                       'Image': "Export a plot to SVG/HTML",
                       'Import': "DICOM import wizard",
                       'Settings': "User Settings",
                       'Database': "Database Administrator Tools",
                       'ROI Map': "Define ROI name aliases"}

        for key in self.toolbar_keys:
            bitmap = wx.Bitmap(ICONS[key], wx.BITMAP_TYPE_ANY)
            if is_windows() or is_linux():
                bitmap = scale_bitmap(bitmap, 30, 30)
            self.frame_toolbar.AddTool(self.toolbar_ids[key], key, bitmap,
                                       wx.NullBitmap, wx.ITEM_NORMAL, description[key], "")

            if key in {'Close', 'Image', 'ROI Map'}:
                separator_count = 4 if is_windows() else 1  # MSW separator is very thin compared to mac
                for _ in range(separator_count):
                    self.frame_toolbar.AddSeparator()

        self.Bind(wx.EVT_TOOL, self.on_save, id=self.toolbar_ids['Save'])
        self.Bind(wx.EVT_TOOL, self.on_open, id=self.toolbar_ids['Open'])
        self.Bind(wx.EVT_TOOL, self.on_export, id=self.toolbar_ids['Export'])
        self.Bind(wx.EVT_TOOL, self.on_export_figure, id=self.toolbar_ids['Image'])
        self.Bind(wx.EVT_TOOL, self.on_toolbar_database, id=self.toolbar_ids['Database'])
        self.Bind(wx.EVT_TOOL, self.on_toolbar_settings, id=self.toolbar_ids['Settings'])
        self.Bind(wx.EVT_TOOL, self.on_toolbar_roi_map, id=self.toolbar_ids['ROI Map'])
        self.Bind(wx.EVT_TOOL, self.on_close, id=self.toolbar_ids['Close'])
        self.Bind(wx.EVT_TOOL, self.on_toolbar_import, id=self.toolbar_ids['Import'])

    def __add_menubar(self):

        self.frame_menubar = wx.MenuBar()

        file_menu = wx.Menu()
        menu_open = file_menu.Append(wx.ID_OPEN, '&Open\tCtrl+O')
        menu_import = file_menu.Append(wx.ID_OPEN, '&Import DICOM\tCtrl+I')
        menu_save = file_menu.Append(wx.ID_ANY, '&Save\tCtrl+S')
        menu_close = file_menu.Append(wx.ID_ANY, '&Close')

        load_model = wx.Menu()
        load_model_mvr = load_model.Append(wx.ID_ANY, 'Multi-Variable Regression')
        load_model_ml = load_model.Append(wx.ID_ANY, 'Machine Learning')

        export = wx.Menu()
        export_csv = export.Append(wx.ID_ANY, 'Data to csv\tCtrl+E')
        export_figure = export.Append(wx.ID_ANY, 'Plot to file')
        file_menu.AppendSeparator()

        qmi = file_menu.Append(wx.ID_ANY, '&Quit\tCtrl+Q')

        self.data_menu = wx.Menu()

        self.data_views = {key: None for key in ['DVHs', 'Plans', 'Rxs', 'Beams']}
        menu_db_admin = self.data_menu.Append(wx.ID_ANY, 'Database Administrator')
        self.data_menu.AppendSubMenu(load_model, 'Load &Model')
        self.data_menu.AppendSubMenu(export, '&Export')
        self.data_menu.AppendSeparator()
        self.data_menu_items = {'DVHs': self.data_menu.Append(wx.ID_ANY, 'Show DVHs\tCtrl+1'),
                                'Plans': self.data_menu.Append(wx.ID_ANY, 'Show Plans\tCtrl+2'),
                                'Rxs': self.data_menu.Append(wx.ID_ANY, 'Show Rxs\tCtrl+3'),
                                'Beams': self.data_menu.Append(wx.ID_ANY, 'Show Beams\tCtrl+4'),
                                'StatsData1': self.data_menu.Append(wx.ID_ANY, 'Show Stats Data: Group 1\tCtrl+5'),
                                'StatsData2': self.data_menu.Append(wx.ID_ANY, 'Show Stats Data: Group 2\tCtrl+6')}

        settings_menu = wx.Menu()
        menu_pref = settings_menu.Append(wx.ID_PREFERENCES)
        menu_sql = settings_menu.Append(wx.ID_ANY, '&Database Connection\tCtrl+D')
        menu_roi_map = settings_menu.Append(wx.ID_ANY, '&ROI Map\tCtrl+R')

        # plan_eval_menu = wx.Menu()
        # menu_protocol_editor = plan_eval_menu.Append(wx.ID_ANY, 'Protocol Editor')

        help_menu = wx.Menu()
        menu_github = help_menu.Append(wx.ID_ANY, 'GitHub Page')
        menu_dvha_edit = help_menu.Append(wx.ID_ANY, 'DVHA DICOM Editor')
        menu_python_libraries = help_menu.Append(wx.ID_ANY, 'Python Libraries Used')
        menu_report_issue = help_menu.Append(wx.ID_ANY, 'Report an Issue')
        menu_about = help_menu.Append(wx.ID_ANY, '&About')

        self.Bind(wx.EVT_MENU, self.on_quit, qmi)
        self.Bind(wx.EVT_MENU, self.on_open, menu_open)
        self.Bind(wx.EVT_MENU, self.on_toolbar_import, menu_import)
        self.Bind(wx.EVT_MENU, self.on_load_mvr_model, load_model_mvr)
        self.Bind(wx.EVT_MENU, self.on_load_ml_model, load_model_ml)
        self.Bind(wx.EVT_MENU, self.on_close, menu_close)
        self.Bind(wx.EVT_MENU, self.on_export, export_csv)
        self.Bind(wx.EVT_MENU, self.on_export_figure, export_figure)
        self.Bind(wx.EVT_MENU, self.on_save, menu_save)
        self.Bind(wx.EVT_MENU, self.on_pref, menu_pref)
        self.Bind(wx.EVT_MENU, self.on_githubpage, menu_github)
        self.Bind(wx.EVT_MENU, self.on_report_issue, menu_report_issue)
        self.Bind(wx.EVT_MENU, self.on_dvha_edit, menu_dvha_edit)
        self.Bind(wx.EVT_MENU, PythonLibraries, menu_python_libraries)
        self.Bind(wx.EVT_MENU, About, menu_about)
        self.Bind(wx.EVT_MENU, self.on_sql, menu_sql)
        self.Bind(wx.EVT_MENU, self.on_toolbar_roi_map, menu_roi_map)
        if is_mac():
            menu_user_settings = settings_menu.Append(wx.ID_ANY, '&Preferences\tCtrl+,')
            self.Bind(wx.EVT_MENU, self.on_pref, menu_user_settings)

        self.Bind(wx.EVT_MENU, self.on_toolbar_database, menu_db_admin)
        self.Bind(wx.EVT_MENU, self.on_view_dvhs, self.data_menu_items['DVHs'])
        self.Bind(wx.EVT_MENU, self.on_view_plans, self.data_menu_items['Plans'])
        self.Bind(wx.EVT_MENU, self.on_view_rxs, self.data_menu_items['Rxs'])
        self.Bind(wx.EVT_MENU, self.on_view_beams, self.data_menu_items['Beams'])
        self.Bind(wx.EVT_MENU, self.on_view_stats_data_1, self.data_menu_items['StatsData1'])
        self.Bind(wx.EVT_MENU, self.on_view_stats_data_2, self.data_menu_items['StatsData2'])

        # self.Bind(wx.EVT_MENU, self.on_protocol_editor, menu_protocol_editor)

        self.frame_menubar.Append(file_menu, '&File')
        self.frame_menubar.Append(self.data_menu, '&Data')
        self.frame_menubar.Append(settings_menu, '&Settings')
        # self.frame_menubar.Append(plan_eval_menu, '&Plan Evaluation')
        self.frame_menubar.Append(help_menu, '&Help')
        self.SetMenuBar(self.frame_menubar)

    def __add_layout_objects(self):
        self.button_categorical = {'add': wx.Button(self, wx.ID_ANY, "Add Filter"),
                                   'del': wx.Button(self, wx.ID_ANY, "Delete Selected"),
                                   'edit': wx.Button(self, wx.ID_ANY, "Edit Selected")}
        self.button_numerical = {'add': wx.Button(self, wx.ID_ANY, "Add Filter"),
                                 'del': wx.Button(self, wx.ID_ANY, "Delete Selected"),
                                 'edit': wx.Button(self, wx.ID_ANY, "Edit Selected")}

        self.table_categorical = wx.ListCtrl(self, wx.ID_ANY, style=wx.BORDER_SUNKEN | wx.LC_REPORT)
        self.table_numerical = wx.ListCtrl(self, wx.ID_ANY, style=wx.BORDER_SUNKEN | wx.LC_REPORT)

        self.radio_button_query_group = wx.RadioBox(self, wx.ID_ANY, 'Query Group', choices=['1', '2'])
        self.button_query_execute = wx.Button(self, wx.ID_ANY, "Query and Retrieve Group 1")

        self.notebook_main_view = wx.Notebook(self, wx.ID_ANY)
        self.tab_keys = ['Welcome', 'DVHs', 'Endpoints', 'Rad Bio', 'Time Series',
                         'Correlation', 'Regression', 'Control Chart']
        self.notebook_tab = {key: wx.Panel(self.notebook_main_view, wx.ID_ANY) for key in self.tab_keys}

        self.text_summary = wx.StaticText(self, wx.ID_ANY, "", style=wx.ALIGN_LEFT)

    def __bind_layout_objects(self):
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_item_select_categorical, self.table_categorical)
        self.Bind(wx.EVT_BUTTON, self.add_row_categorical, id=self.button_categorical['add'].GetId())
        self.Bind(wx.EVT_BUTTON, self.del_row_categorical, id=self.button_categorical['del'].GetId())
        self.Bind(wx.EVT_BUTTON, self.edit_row_categorical, id=self.button_categorical['edit'].GetId())
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.doubleclick_categorical, self.table_categorical)

        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_item_select_numerical, self.table_numerical)
        self.Bind(wx.EVT_BUTTON, self.add_row_numerical, id=self.button_numerical['add'].GetId())
        self.Bind(wx.EVT_BUTTON, self.del_row_numerical, id=self.button_numerical['del'].GetId())
        self.Bind(wx.EVT_BUTTON, self.edit_row_numerical, id=self.button_numerical['edit'].GetId())
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.doubleclick_numerical, self.table_numerical)

        self.Bind(wx.EVT_RADIOBOX, self.on_group_select, id=self.radio_button_query_group.GetId())
        self.Bind(wx.EVT_BUTTON, self.exec_query_button, id=self.button_query_execute.GetId())

        self.Bind(wx.EVT_SIZE, self.on_resize)

    def __set_properties(self):
        self.SetTitle("DVH Analytics")

        self.frame_toolbar.Realize()

        widths = [180, 150, 80]
        self.table_categorical.AppendColumn("Category1", format=wx.LIST_FORMAT_LEFT, width=widths[0])
        self.table_categorical.AppendColumn("Category2", format=wx.LIST_FORMAT_LEFT, width=widths[1])
        self.table_categorical.AppendColumn("Filter Type", format=wx.LIST_FORMAT_LEFT, width=widths[2])
        self.data_table_categorical.widths = widths

        widths = [150, 90, 90, 80]
        self.table_numerical.AppendColumn("Category", format=wx.LIST_FORMAT_LEFT, width=widths[0])
        self.table_numerical.AppendColumn("Min", format=wx.LIST_FORMAT_LEFT, width=widths[1])
        self.table_numerical.AppendColumn("Max", format=wx.LIST_FORMAT_LEFT, width=widths[2])
        self.table_numerical.AppendColumn("Filter Type", format=wx.LIST_FORMAT_LEFT, width=widths[3])
        self.data_table_numerical.widths = widths

    def __set_tooltips(self):
        self.button_categorical['add'].SetToolTip("Add a categorical data filter.")
        self.button_categorical['del'].SetToolTip("Delete the currently selected category filter.")
        self.button_categorical['edit'].SetToolTip("Edit the currently selected category filter.")
        self.button_numerical['add'].SetToolTip("Add a numerical data filter.")
        self.button_numerical['del'].SetToolTip("Delete the currently selected numerical data filter.")
        self.button_numerical['edit'].SetToolTip("Edit the currently selected data filter.")
        self.button_query_execute.SetToolTip("Query the database with the filters entered above. At least one "
                                             "filter must be added.")

    def __add_notebook_frames(self):
        self.plot = PlotStatDVH(self.notebook_tab['DVHs'], self.group_data, self.options)
        self.time_series = TimeSeriesFrame(self.notebook_tab['Time Series'], self.group_data, self.options)
        self.correlation = CorrelationFrame(self.notebook_tab['Correlation'], self.group_data, self.options)
        self.regression = RegressionFrame(self.notebook_tab['Regression'], self.group_data, self.options, self)
        self.control_chart = ControlChartFrame(self.notebook_tab['Control Chart'], self.group_data, self.options)
        self.radbio = RadBioFrame(self.notebook_tab['Rad Bio'], self.group_data, self.time_series, self.regression,
                                  self.control_chart)
        self.endpoint = EndpointFrame(self.notebook_tab['Endpoints'], self.group_data, self.time_series, self.regression,
                                      self.control_chart)

    def __do_layout(self):
        sizer_summary = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, "Summary"), wx.HORIZONTAL)
        sizer_query_numerical = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, "Query by Numerical Data"),
                                                  wx.VERTICAL)

        sizer_query_categorical = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, "Query by Categorical Data"),
                                                    wx.VERTICAL)

        sizer_categorical_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_numerical_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_query_exec_buttons = wx.BoxSizer(wx.HORIZONTAL)

        for key in ['add', 'del', 'edit']:
            sizer_categorical_buttons.Add(self.button_categorical[key], 0, wx.ALL, 5)
            sizer_numerical_buttons.Add(self.button_numerical[key], 0, wx.ALL, 5)

        sizer_query_categorical.Add(sizer_categorical_buttons, 0, wx.ALL | wx.EXPAND, 5)
        sizer_query_categorical.Add(self.table_categorical, 1, wx.ALL | wx.EXPAND, 10)

        sizer_query_numerical.Add(sizer_numerical_buttons, 0, wx.ALL | wx.EXPAND, 5)
        sizer_query_numerical.Add(self.table_numerical, 1, wx.ALL | wx.EXPAND, 10)

        sizer_summary.Add(self.text_summary)

        panel_left = wx.BoxSizer(wx.VERTICAL)
        panel_left.Add(sizer_query_categorical, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.SHAPED | wx.TOP, 5)
        panel_left.Add(sizer_query_numerical, 0, wx.BOTTOM | wx.EXPAND | wx.LEFT | wx.RIGHT | wx.SHAPED, 5)
        sizer_query_exec_buttons.Add(self.radio_button_query_group, 0, 0, 0)
        sizer_query_exec_buttons.Add(self.button_query_execute, 1, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 10)
        panel_left.Add(sizer_query_exec_buttons, 0, wx.EXPAND | wx.ALIGN_CENTER | wx.RIGHT | wx.LEFT, 5)
        panel_left.Add(sizer_summary, 1, wx.ALL | wx.EXPAND, 5)

        bitmap_logo = wx.StaticBitmap(self.notebook_tab['Welcome'], wx.ID_ANY,
                                      wx.Bitmap(LOGO_PATH, wx.BITMAP_TYPE_ANY))
        text_welcome = wx.StaticText(self.notebook_tab['Welcome'], wx.ID_ANY,
                                     "\n\nWelcome to DVH Analytics.\nIf you already have a database built, design "
                                     "a query with the filters to the left.\n\n\n\nDVH Analytics is a software "
                                     "application to help radiation oncology departments build an in-house database "
                                     "of treatment planning data for the purpose of historical comparisons and "
                                     "statistical analysis. This code is still in development. Please contact the "
                                     "developer if you are interested in testing or collaborating.\n\nThe application "
                                     "builds a SQL database of DVHs and various planning parameters from DICOM files "
                                     "(i.e., Plan, Structure, Dose). Since the data is extracted directly from DICOM "
                                     "files, we intend to accommodate an array of treatment planning system vendors.",
                                     style=wx.ALIGN_CENTER)

        text_welcome_size = get_window_size(0.417, 0.476)
        text_welcome.SetMinSize(text_welcome_size)
        text_welcome.Wrap(text_welcome_size[1])

        sizer_welcome = wx.BoxSizer(wx.VERTICAL)
        sizer_welcome.Add(bitmap_logo, 0, wx.ALIGN_CENTER | wx.LEFT | wx.RIGHT | wx.TOP, 100)
        sizer_welcome.Add(text_welcome, 0, wx.ALIGN_CENTER | wx.ALL, 25)
        self.notebook_tab['Welcome'].SetSizer(sizer_welcome)

        self.sizer_dvhs.Add(self.plot.layout, 1, wx.EXPAND | wx.ALL, 25)
        self.notebook_tab['DVHs'].SetSizer(self.sizer_dvhs)

        sizer_endpoint = wx.BoxSizer(wx.VERTICAL)
        sizer_endpoint.Add(self.endpoint.layout, 0, wx.ALL, 25)
        self.notebook_tab['Endpoints'].SetSizer(sizer_endpoint)

        sizer_rad_bio = wx.BoxSizer(wx.VERTICAL)
        sizer_rad_bio.Add(self.radbio.layout, 0, wx.ALL, 25)
        self.notebook_tab['Rad Bio'].SetSizer(sizer_rad_bio)

        sizer_time_series = wx.BoxSizer(wx.VERTICAL)
        sizer_time_series.Add(self.time_series.layout, 1, wx.EXPAND | wx.ALL, 25)
        self.notebook_tab['Time Series'].SetSizer(sizer_time_series)

        sizer_correlation = wx.BoxSizer(wx.VERTICAL)
        sizer_correlation.Add(self.correlation.layout, 1, wx.EXPAND | wx.ALL, 25)
        self.notebook_tab['Correlation'].SetSizer(sizer_correlation)

        sizer_regression = wx.BoxSizer(wx.VERTICAL)
        sizer_regression.Add(self.regression.layout, 1, wx.EXPAND | wx.ALL, 25)
        self.notebook_tab['Regression'].SetSizer(sizer_regression)

        sizer_control_chart = wx.BoxSizer(wx.VERTICAL)
        sizer_control_chart.Add(self.control_chart.layout, 1, wx.EXPAND | wx.ALL, 25)
        self.notebook_tab['Control Chart'].SetSizer(sizer_control_chart)

        for key in self.tab_keys:
            self.notebook_main_view.AddPage(self.notebook_tab[key], key)

        sizer_main = wx.BoxSizer(wx.VERTICAL)
        sizer_main_wrapper = wx.BoxSizer(wx.HORIZONTAL)
        hbox_main = wx.BoxSizer(wx.HORIZONTAL)
        hbox_main.Add(panel_left, 0, wx.BOTTOM | wx.EXPAND | wx.LEFT | wx.TOP, 5)
        hbox_main.Add(self.notebook_main_view, 1, wx.BOTTOM | wx.EXPAND | wx.RIGHT | wx.TOP, 5)
        sizer_main_wrapper.Add(hbox_main, 1, wx.EXPAND, 0)
        sizer_main.Add(sizer_main_wrapper, 1, wx.EXPAND, 0)

        self.SetSizer(sizer_main)
        self.Layout()

        self.SetSize(get_window_size(0.833, 0.875))

        self.Center()

    def __enable_notebook_tabs(self):
        for key in self.tab_keys:
            self.notebook_tab[key].Enable()
        self.__enable_initial_buttons_in_tabs()

    def __disable_notebook_tabs(self):
        for key in self.tab_keys:
            if key != 'Welcome':
                self.notebook_tab[key].Disable()

    def __enable_initial_buttons_in_tabs(self):
        self.endpoint.enable_initial_buttons()
        self.radbio.enable_initial_buttons()
        self.time_series.enable_initial_buttons()

    def __disable_add_filter_buttons(self):
        self.button_categorical['add'].Disable()
        self.button_numerical['add'].Disable()

    def __enable_add_filter_buttons(self):
        self.button_categorical['add'].Enable()
        self.button_numerical['add'].Enable()

    def enable_query_buttons(self, query_type):
        for key in ['del', 'edit']:
            {'categorical': self.button_categorical[key],
             'numerical': self.button_numerical[key]}[query_type].Enable()

    def disable_query_buttons(self, query_type):
        for key in ['del', 'edit']:
            {'categorical': self.button_categorical[key],
             'numerical': self.button_numerical[key]}[query_type].Disable()

    def update_all_query_buttons(self):
        tables = {'numerical': self.data_table_numerical, 'categorical': self.data_table_categorical}
        for key, table in tables.items():
            if table.data is not None:
                [self.disable_query_buttons, self.enable_query_buttons][table.row_count > 0](key)
            else:
                self.disable_query_buttons(key)

        if self.data_table_numerical.row_count + self.data_table_categorical.row_count > 0:
            self.button_query_execute.Enable()
        else:
            self.button_query_execute.Disable()

        # Force user to populate group 1 first
        if self.selected_group == 2 and self.group_data[1]['dvh'] is None:
            self.button_query_execute.Disable()

    def __catch_failed_sql_connection_on_app_launch(self):
        if self.options.DB_TYPE == 'pgsql':
            if not echo_sql_db():
                wx.MessageBox('Invalid credentials!', 'Echo SQL Database', wx.OK | wx.ICON_WARNING)
                self.on_sql()
        else:  # if using sqlite
            initialize_db()

    def __do_subscribe(self):
        pub.subscribe(self.raise_error_dialog, "import_status_raise_error")

    def raise_error_dialog(self, msg):
        MemoryErrorDialog(self, msg)

    # --------------------------------------------------------------------------------------------------------------
    # Menu bar event functions
    # --------------------------------------------------------------------------------------------------------------
    def on_save(self, evt):
        if self.save_data:
            dlg = wx.FileDialog(self, "Save your session data to file", "", wildcard='*.dvha',
                                style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
            dlg.SetDirectory(DATA_DIR)
            if dlg.ShowModal() == wx.ID_OK:
                self.save_data_obj()
                save_object_to_file(self.save_data, dlg.GetPath())
            dlg.Destroy()
        else:
            wx.MessageBox('There is no data to save. Please query/open some data first.', 'Save Error',
                          wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING)

    def on_open(self, evt):
        dlg = wx.FileDialog(self, "Open saved data", "", wildcard='*.dvha',
                            style=wx.FD_FILE_MUST_EXIST | wx.FD_OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.close()
            self.load_data_obj(dlg.GetPath())

        dlg.Destroy()

    def on_load_mvr_model(self, *evt):
        with wx.FileDialog(self, "Load a multi-variable linear regression model", "", wildcard='*.mvr',
                           style=wx.FD_FILE_MUST_EXIST | wx.FD_OPEN) as dlg:
            dlg.SetDirectory(MODELS_DIR)
            if dlg.ShowModal() == wx.ID_OK:
                model_file_path = dlg.GetPath()
                dlg.Destroy()
                self.loaded_mvr_ml_frames.append(LoadMultiVarModelFrame(model_file_path, self.group_data,
                                                                        self.selected_group, self.options))

    def on_load_ml_model(self, *evt):
        if self.group_data[self.selected_group]['stats_data']:
            self.loaded_mvr_ml_frames.append(MachineLearningModelViewer(self, self.group_data,
                                                                        self.selected_group, self.options))
        else:
            wx.MessageBox('No data as been queried for Group %s.' % self.selected_group, 'Error',
                          wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING)

    @property
    def selected_group(self):
        return self.radio_button_query_group.GetSelection() + 1

    def save_data_obj(self):
        self.save_data['group_data'] = self.group_data
        self.save_data['query_filters'] = self.query_filters
        self.save_data['time_stamp'] = datetime.now()
        self.save_data['version'] = DefaultOptions().VERSION
        # data_table_categorical and data_table_numerical saved after query to ensure these data reflect
        # the rest of the saved data
        self.save_data['endpoint'] = self.endpoint.get_save_data()
        self.save_data['time_series'] = self.time_series.get_save_data()
        self.save_data['radbio'] = self.radbio.get_save_data()
        self.save_data['regression'] = self.regression.get_save_data()
        self.save_data['control_chart'] = self.control_chart.get_save_data()

    def load_data_obj(self, abs_file_path):
        self.save_data = load_object_from_file(abs_file_path)
        self.group_data = self.save_data['group_data']

        # .load_save_data loses column widths?
        self.radio_button_query_group.SetSelection(0)
        self.data_table_categorical.load_save_data(self.save_data['main_categorical_1'])
        self.data_table_numerical.load_save_data(self.save_data['main_numerical_1'])

        self.control_chart.load_save_data(self.save_data['control_chart'])

        self.radio_button_query_group.SetSelection(0)
        self.exec_query(load_saved_dvh_data=True, group=1)

        if 'main_categorical_2' in self.save_data.keys():
            self.radio_button_query_group.SetSelection(1)
            self.on_group_select()
            self.data_table_categorical.load_save_data(self.save_data['main_categorical_2'])
            self.data_table_numerical.load_save_data(self.save_data['main_numerical_2'])
            self.update_all_query_buttons()

            self.exec_query(load_saved_dvh_data=True, group=2)

        self.update_all_query_buttons()

        self.endpoint.load_save_data(self.save_data['endpoint'])
        if self.endpoint.has_data:
            self.endpoint.enable_buttons()

        self.radbio.load_save_data(self.save_data['radbio'])

        self.endpoint.update_endpoints_in_dvh()

        self.group_data[1]['stats_data'].update_endpoints_and_radbio()
        if self.group_data[2]['stats_data']:
            self.group_data[2]['stats_data'].update_endpoints_and_radbio()

        self.time_series.load_save_data(self.save_data['time_series'])
        self.time_series.update_plot()

        self.regression.update_combo_box_choices()
        self.regression.load_save_data(self.save_data['regression'])

        self.control_chart.update_combo_box_y_choices()

    def on_toolbar_settings(self, evt):
        self.on_pref()

    def on_toolbar_import(self, evt):
        self.check_db_then_call(ImportDicomFrame, 'import', self.roi_map, self.options)

    def on_toolbar_database(self, evt):
        self.check_db_then_call(DatabaseEditorFrame, 'database', self.roi_map, self.options)

    def on_toolbar_roi_map(self, evt):
        self.check_db_then_call(ROIMapFrame, 'roi_map', self.roi_map)

    def check_db_then_call(self, func, window_type, *parameters):
        if not echo_sql_db():
            self.on_sql()

        if echo_sql_db():
            if self.tool_bar_windows[window_type]:
                self.tool_bar_windows[window_type].Raise()
            else:
                self.tool_bar_windows[window_type] = func(*parameters)
        else:
            wx.MessageBox('Connection to SQL database could not be established.', 'Connection Error',
                          wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING)

    # --------------------------------------------------------------------------------------------------------------
    # Query event functions
    # --------------------------------------------------------------------------------------------------------------
    def on_item_select_categorical(self, evt):
        self.selected_index_categorical = self.table_categorical.GetFirstSelected()

    def on_item_select_numerical(self, evt):
        self.selected_index_numerical = self.table_numerical.GetFirstSelected()

    def add_row_categorical(self, evt):
        query_dlg(self, 'categorical')
        self.button_query_execute.Enable()
        self.update_all_query_buttons()

    def add_row_numerical(self, evt):
        query_dlg(self, 'numerical')
        self.update_all_query_buttons()

    def edit_row_categorical(self, *args):
        if self.selected_index_categorical is not None:
            query_dlg(self, 'categorical', set_values=True)

    def edit_row_numerical(self, *args):
        if self.selected_index_numerical is not None:
            query_dlg(self, 'numerical', set_values=True)

    def doubleclick_categorical(self, evt):
        self.selected_index_categorical = self.table_categorical.GetFirstSelected()
        self.edit_row_categorical()

    def doubleclick_numerical(self, evt):
        self.selected_index_numerical = self.table_numerical.GetFirstSelected()
        self.edit_row_numerical()

    def del_row_categorical(self, evt):
        self.data_table_categorical.delete_row(self.selected_index_categorical)
        self.selected_index_categorical = None
        self.update_all_query_buttons()

    def del_row_numerical(self, evt):
        self.data_table_numerical.delete_row(self.selected_index_numerical)
        self.selected_index_numerical = None
        self.update_all_query_buttons()

    def exec_query_button(self, evt):
        self.exec_query()

    def exec_query(self, load_saved_dvh_data=False, group=None):
        wx.BeginBusyCursor()
        if group is not None:
            self.radio_button_query_group.SetSelection(group - 1)
        group = self.selected_group

        # TODO: retain group 1 endpoint defs after query of group 2
        self.endpoint.clear_data()

        if group == 1:
            self.plot.clear_plot()
            self.time_series.clear_data()
            self.regression.clear(self.group_data)
            self.control_chart.clear_data()
            self.radbio.clear_data()

        if not load_saved_dvh_data:
            try:
                uids, dvh_str = self.get_query()
                self.group_data[group]['dvh'] = \
                    DVH(dvh_condition=dvh_str, uid=uids, dvh_bin_width=self.options.dvh_bin_width)
            except MemoryError:
                msg = "Querying memory error. Try querying less data. At least %s DVHs returned.\n"\
                      "NOTE: Threshold of this error is dependent on your computer." % self.group_data[group]['dvh'].count
                MemoryErrorDialog(self, msg)
                self.close()
                return

        count = self.group_data[group]['dvh'].count
        if count > 1:
            try:
                self.endpoint.update_dvh(self.group_data)
                self.set_summary_text(group)
                self.plot.update_plot(self.group_data[1]['dvh'], dvh_2=self.group_data[2]['dvh'])
                wx.EndBusyCursor()
                self.update_data(load_saved_dvh_data=load_saved_dvh_data, group_2_only=bool(group-1))

                if group == 1:
                    self.notebook_main_view.SetSelection(1)
                    self.__enable_notebook_tabs()

                self.save_data['main_categorical_%s' % group] = self.data_table_categorical.get_save_data()
                self.save_data['main_numerical_%s' % group] = self.data_table_numerical.get_save_data()

                if group == 2:
                    self.regression.group = 2
                    self.control_chart.group = 2
                    self.update_stats_data_plots()

            except PlottingMemoryError as e:
                wx.EndBusyCursor()
                self.on_plotting_memory_error(str(e))
        else:
            wx.EndBusyCursor()
            msg = "%s DVHs returned. Please modify query or import more data." % ['Less than 2', 'No'][count == 0]
            wx.MessageBox(msg, 'Query Error', wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING)
            self.group_data[group]['dvh'] = None

    def get_query(self):

        # Used to accumulate lists of query strings for each table
        # Will assume each item in list is complete query for that SQL column
        queries = {'Plans': [], 'Rxs': [], 'Beams': [], 'DVHs': []}

        # Used to group queries by variable, will combine all queries of same variable with an OR operator
        # e.g., queries_by_sql_column['Plans'][key] = list of strings, where key is sql column
        queries_by_sql_column = {'Plans': {}, 'Rxs': {}, 'Beams': {}, 'DVHs': {}}

        # Categorical filter
        if self.data_table_categorical.row_count:
            for i, category in enumerate(self.data_table_categorical.data['category_1']):
                table = self.categorical_columns[category]['table']
                col = self.categorical_columns[category]['var_name']
                value = self.data_table_categorical.data['category_2'][i]
                if col not in queries_by_sql_column[table]:
                    queries_by_sql_column[table][col] = []
                operator = ['=', '!='][{'Include': 0, 'Exclude': 1}[self.data_table_categorical.data['Filter Type'][i]]]
                queries_by_sql_column[table][col].append("%s %s '%s'" % (col, operator, value))

        # Range filter
        if self.data_table_numerical.row_count:
            for i, category in enumerate(self.data_table_numerical.data['category']):
                table = self.numerical_columns[category]['table']
                col = self.numerical_columns[category]['var_name']
                value_low = self.data_table_numerical.data['min'][i]
                value_high = self.data_table_numerical.data['max'][i]
                if 'date' in col:
                    value_low = "'%s'" % value_low
                    value_high = "'%s'" % value_high
                    if self.options.DB_TYPE == 'pgsql':
                        value_low = value_low + '::date'
                        value_high = value_high + '::date'
                if col not in queries_by_sql_column[table]:
                    queries_by_sql_column[table][col] = []
                operator = ['BETWEEN', 'NOT BETWEEN'][
                    {'Include': 0, 'Exclude': 1}[self.data_table_numerical.data['Filter Type'][i]]]
                queries_by_sql_column[table][col].append("%s %s %s AND %s" % (col, operator, value_low, value_high))

        for table in queries:
            for col in queries_by_sql_column[table]:
                queries[table].append("(%s)" % ' OR '.join(queries_by_sql_column[table][col]))
            queries[table] = ' AND '.join(queries[table])

        uids = get_study_instance_uids(plans=queries['Plans'], rxs=queries['Rxs'], beams=queries['Beams'])['common']

        return uids, queries['DVHs']

    def update_data(self, load_saved_dvh_data=False, group_2_only=False):
        wx.BeginBusyCursor()
        tables = ['Plans', 'Rxs', 'Beams']
        for grp, grp_data in self.group_data.items():
            if not(grp == 1 and group_2_only) or grp == 2:
                if hasattr(grp_data['dvh'], 'study_instance_uid'):
                    if not load_saved_dvh_data:
                        condition_str = "study_instance_uid in ('%s')" % "','".join(
                           grp_data['dvh'].study_instance_uid)
                        grp_data['data'] = {key: QuerySQL(key, condition_str) for key in tables}
                        grp_data['stats_data'] = StatsData(grp_data['dvh'], grp_data['data'], group=grp)
                else:
                    grp_data['data'] = {key: None for key in tables}
                    grp_data['stats_data'] = None

        if self.group_data[2]['stats_data']:
            sync_variables_in_stats_data_objects(self.group_data[1]['stats_data'],
                                                 self.group_data[2]['stats_data'])
        self.time_series.update_data(self.group_data)
        self.control_chart.update_data(self.group_data)
        self.correlation.set_data(self.group_data)
        self.regression.update_combo_box_choices()
        self.radbio.update_dvh_data(self.group_data)

        wx.EndBusyCursor()

    # --------------------------------------------------------------------------------------------------------------
    # Menu bar event functions
    # --------------------------------------------------------------------------------------------------------------
    def close_windows(self):
        for view in self.data_views.values():
            if hasattr(view, 'Destroy'):
                try:
                    view.Destroy()
                except RuntimeError:
                    pass

        for window in self.tool_bar_windows.values():
            if window and hasattr(window, 'Destroy'):
                window.Destroy()

        self.regression.close_mvr_frames()

    def on_quit(self, evt):
        self.close_windows()
        self.Destroy()

    def on_close(self, *evt):
        if self.group_data[1]['dvh']:
            dlg = wx.MessageDialog(self, "Clear all data and plots?", caption='Close',
                                   style=wx.YES | wx.NO | wx.NO_DEFAULT | wx.CENTER | wx.ICON_EXCLAMATION)
            dlg.Center()
            res = dlg.ShowModal()
            if res == wx.ID_YES:
                self.close()
                self.radio_button_query_group.SetSelection(0)
            dlg.Destroy()

    def close(self):
        self.group_data = {1: {'dvh': None,
                               'data': {key: None for key in ['Plans', 'Beams', 'Rxs']},
                               'stats_data': None},
                           2: {'dvh': None,
                               'data': {key: None for key in ['Plans', 'Beams', 'Rxs']},
                               'stats_data': None}}
        self.data_table_categorical.delete_all_rows()
        self.data_table_numerical.delete_all_rows()
        self.plot.clear_plot()
        self.endpoint.clear_data()
        self.radbio.clear_data()
        self.time_series.clear_data()
        self.notebook_main_view.SetSelection(0)
        self.text_summary.SetLabelText("")
        self.__disable_notebook_tabs()
        self.disable_query_buttons('categorical')
        self.disable_query_buttons('numerical')
        self.button_query_execute.Disable()
        self.time_series.initialize_y_axis_options()
        self.regression.clear(self.group_data)
        self.control_chart.initialize_y_axis_options()
        self.control_chart.plot.clear_plot()
        self.control_chart.group = 1
        self.close_windows()
        self.reset_query_filters()

    def on_export(self, evt):
        if self.group_data[1]['dvh'] is not None:
            ExportCSVDialog(self)
        else:
            wx.MessageBox('There is no data to export! Please query some data first.', 'Export Error',
                          wx.OK | wx.ICON_WARNING)

    @staticmethod
    def on_githubpage(evt):
        webbrowser.open_new_tab("http://dvhanalytics.com/")

    @staticmethod
    def on_dvha_edit(evt):
        webbrowser.open_new_tab("https://github.com/cutright/DVHA-DICOM-Editor")

    @staticmethod
    def on_report_issue(evt):
        webbrowser.open_new_tab("https://github.com/cutright/DVH-Analytics/issues")

    def on_pref(self, *args):
        if self.user_settings is None:
            self.user_settings = UserSettings(self)
            self.user_settings.Show()
        else:
            self.user_settings.Raise()

    def on_sql(self, *args):
        SQLSettingsDialog(self.options)
        [self.__disable_add_filter_buttons, self.__enable_add_filter_buttons][echo_sql_db()]()

    def on_export_figure(self, evt):
        if self.save_data:
            if self.export_figure is None:
                self.export_figure = ExportFigure(self)
                self.export_figure.Show()
            else:
                self.export_figure.Raise()
        else:
            wx.MessageBox('There is no data to save. Please query/open some data first.', 'Save Error',
                          wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING)

    def on_view_dvhs(self, evt):
        self.view_table_data('DVHs')

    def on_view_plans(self, evt):
        self.view_table_data('Plans')

    def on_view_rxs(self, evt):
        self.view_table_data('Rxs')

    def on_view_beams(self, evt):
        self.view_table_data('Beams')

    def on_view_stats_data_1(self, evt):
        self.view_table_data('StatsData1')

    def on_view_stats_data_2(self, evt):
        self.view_table_data('StatsData2')

    def view_table_data(self, key):
        if key == 'DVHs':
            data = {grp: self.group_data[grp]['dvh'] for grp in [1, 2]}
        elif 'StatsData' in key:
            data = {grp: self.group_data[grp]['stats_data'] for grp in [1, 2]}
        else:
            data = {grp: self.group_data[grp]['data'][key] for grp in [1, 2]}

        if data[1]:

            if self.get_menu_item_status(key) == 'Show':
                if 'StatsData' in key:
                    group = int(key[-1])
                    if group == 1 or data[2] is not None:
                        self.data_views[key] = StatsDataEditor(self.group_data, group, self.data_menu,
                                                               self.data_menu_items[key].GetId(), self.time_series,
                                                               self.regression, self.control_chart)
                    else:
                        self.no_queried_data_dlg()
                else:
                    if key == 'DVHs':
                        columns = [c for c in data[1].keys]
                    elif key == 'Rxs':
                        columns = ['plan_name', 'fx_dose', 'rx_percent', 'fxs', 'rx_dose', 'fx_grp_number',
                                   'fx_grp_count',
                                   'fx_grp_name', 'normalization_method', 'normalization_object']
                    else:
                        columns = [obj['var_name'] for obj in sql_column_info.values() if obj['table'] == key]

                    for starter_column in ['study_instance_uid', 'mrn']:
                        if starter_column in columns:
                            columns.pop(columns.index(starter_column))
                        columns.insert(0, starter_column)

                    self.data_views[key] = QueriedDataFrame(data, columns, key,
                                                            self.data_menu, self.data_menu_items[key].GetId())
            else:
                self.data_views[key].on_close()
                self.data_views[key] = None
        else:
            self.no_queried_data_dlg()

    def on_protocol_editor(self, *evt):
        ProtocolsEditor(self.roi_map)

    def no_queried_data_dlg(self):
        dlg = wx.MessageDialog(self, 'Please query/open some data first.', 'ERROR!', wx.ICON_ERROR | wx.OK_DEFAULT)
        dlg.ShowModal()
        dlg.Destroy()

    def get_menu_item_status(self, key):
        show_hide = ['Hide', 'Show']['Show' in self.data_menu.GetLabel(self.data_menu_items[key].GetId())]
        return show_hide

    def redraw_plots(self):
        if self.group_data[1]['dvh']:
            self.plot.redraw_plot()
            self.time_series.plot.redraw_plot()
            self.correlation.plot.redraw_plot()
            self.regression.plot.redraw_plot()
            self.control_chart.plot.redraw_plot()

    def apply_plot_options(self):
        self.plot.apply_options()
        self.time_series.plot.apply_options()
        self.correlation.plot.apply_options()
        self.regression.apply_plot_options()  # applies to MVR frames too
        self.control_chart.plot.apply_options()

        for frame in self.loaded_mvr_ml_frames:
            try:
                frame.apply_plot_options()
            except RuntimeError:
                pass

    def update_stats_data_plots(self):
        if self.group_data[1]['dvh']:
            self.time_series.update_plot()
            self.time_series.update_y_axis_options()
            self.regression.update_plot()
            self.control_chart.update_plot()

    def on_resize(self, *evt):
        try:
            self.Refresh()
            self.Layout()
            wx.CallAfter(self.redraw_plots)
        except RuntimeError:
            pass

    def on_plotting_memory_error(self, plot_type):
        plot_type = [' (Plot type: %s)' % plot_type, ''][plot_type is None]
        msg = "Plotting memory error%s. Try querying less data. At least %s DVHs returned.\n" \
              "NOTE: Threshold of this error is dependent on your computer." % (plot_type, self.group_data[1]['dvh'].count)
        MemoryErrorDialog(self, msg)
        self.close()

    def reset_query_filters(self):
        self.query_filters = {key: None for key in [1, 2]}

    def on_group_select(self, *evt):
        group = self.selected_group
        other = 3 - group

        self.button_query_execute.SetLabelText('Query and Retrieve Group %s' % group)

        self.query_filters[other] = {'main_categorical': self.data_table_categorical.get_save_data(),
                                     'main_numerical': self.data_table_numerical.get_save_data()}
        if self.query_filters[group] is not None:
            self.data_table_categorical.load_save_data(self.query_filters[group]['main_categorical'])
            self.data_table_numerical.load_save_data(self.query_filters[group]['main_numerical'])
        else:
            self.data_table_categorical.delete_all_rows()
            self.data_table_numerical.delete_all_rows()

        self.update_all_query_buttons()

        self.set_summary_text(group)

        if self.group_data[2]['stats_data']:
            self.regression.group = group
            self.regression.update_plot()
            self.control_chart.group = group
            self.control_chart.update_plot()

    def set_summary_text(self, group):
        if self.group_data[group]['dvh']:
            text = self.group_data[group]['dvh'].get_summary()
        else:
            text = ''
        self.text_summary.SetLabelText(text)


class MainApp(wx.App):
    def OnInit(self):

        initialize_directories()
        if is_windows():
            from dvha.tools.windows_reg_edit import set_ie_emulation_level, set_ie_lockdown_level
            set_ie_emulation_level()
            set_ie_lockdown_level()

        self.SetAppName('DVH Analytics')
        self.frame = DVHAMainFrame(None, wx.ID_ANY, "")
        set_frame_icon(self.frame)
        self.SetTopWindow(self.frame)
        self.frame.Show()
        return True

    def OnExit(self):
        for window in wx.GetTopLevelWindows():
            wx.CallAfter(window.Close)
        return super().OnExit()


def start():
    app = MainApp(0)
    app.MainLoop()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# models.data_table.py
"""
A class to sync a data object and list_ctrl
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

from copy import deepcopy
import wx
from dvha.tools.utilities import get_selected_listctrl_items, get_sorted_indices


class DataTable:
    """
    This is a helper class containing the UI elements of the list_ctrl.  Adding / Changing data with this class
    will automatically update UI elements.
    """
    def __init__(self, list_ctrl, data=None, columns=None, widths=None, formats=None):
        """
        :param list_ctrl: the list_ctrl in the GUI to be updated with data in this class
        :type list_ctrl: ListCtrl
        :param data: data should be formatted in a dictionary with keys being the column names and values being lists
        :type data: dict
        :param columns: the keys of the data object to be visible in the list_ctrl
        :type columns: list
        :param widths: optionally specify the widths of the columns
        :type widths: list
        :param formats: optionally specify wx Format values (e.g., wx.LIST_FORMAT_LEFT)
        :type formats: list
        """

        self.layout = list_ctrl

        self.sort_indices = None

        self.data = deepcopy(data)
        self.columns = deepcopy(columns)
        self.widths = widths
        if formats:
            self.formats = formats
        else:
            if not self.columns:
                column_length = 0
            else:
                column_length = len(self.columns)
            self.formats = [wx.LIST_FORMAT_LEFT] * column_length
        if data:
            # TODO: Initializing class with duplicates data in view?
            self.set_data(data, columns, formats=formats)
        self.set_data_in_layout()

    def get_save_data(self):
        """
        This function is used to save data within a .dvha file
        :return: a copy of the data, columns, widths, and formats
        :rtype: dict
        """
        return deepcopy({'data': self.data,
                         'columns': self.columns,
                         'widths': self.widths,
                         'formats': self.formats})

    def load_save_data(self, save_data, ignore_layout=False):
        """
        This function is used to load data from saved .dvha file
        :param save_data: output from get_save_data
        :type save_data: dict
        :param ignore_layout: If true, do not update layout
        :type ignore_layout: bool
        """
        self.widths = deepcopy(save_data['widths'])
        self.set_column_widths()
        self.set_data(save_data['data'], save_data['columns'], ignore_layout=ignore_layout)

    def set_data(self, data, columns, formats=None, ignore_layout=False):
        """
        Use this function to update data and properly update the layout
        :param data: data should be formatted in a dictionary with keys being the column names and values being lists
        :type data: dict
        :param columns: the keys of the data object to be visible in the list_ctrl
        :type columns: list
        :param formats: optionally specify wx Format values (e.g., wx.LIST_FORMAT_LEFT)
        :type formats: list
        :param ignore_layout: If true, do not update layout
        :type ignore_layout: bool
        """
        if formats:
            self.formats = formats
        elif columns and len(columns) != len(self.formats):
            self.formats = [wx.LIST_FORMAT_LEFT] * len(columns)

        delete_rows = bool(self.row_count)
        self.data = deepcopy(data)
        self.columns = deepcopy(columns)
        if delete_rows:
            self.delete_all_rows(layout_only=True)

        if not ignore_layout:
            self.set_layout_columns()
            self.set_data_in_layout()

        if self.widths:
            self.set_column_widths()

        self.sort_indices = None  # If len of new data is different than previous, sorting may crash

    def set_layout_columns(self):
        self.layout.DeleteAllColumns()
        for i, col in enumerate(self.columns):
            self.layout.AppendColumn(col, format=self.formats[i])

    @property
    def keys(self):
        return [col for col in self.columns]

    @property
    def column_count(self):
        if self.columns:
            return len(self.columns)
        return 0

    @property
    def row_count(self):
        if self.data:
            return len(self.data[self.columns[0]])
        return 0

    def data_to_list_of_rows(self):
        """
        Convert self.data which is formatted as a dict by columns into a list of rows as needed for list_ctrl
        self.data is formatted this way out of convience for plotting with bokeh
        :return: data in the format of list of rows
        :rtype: list
        """
        if self.data and self.keys:
            return [[self.data[col][row] for col in self.columns] for row in range(self.row_count)]
        else:
            return []

    def add_column(self, column, format=wx.LIST_FORMAT_LEFT):
        """
        Add an empty column to the data and layout
        :param column: name of the column
        :type column: str
        :param format: optionally specify the wx format value (wx.LIST_FORMAT_LEFT by default)
        """
        if self.layout:
            self.layout.AppendColumn(column, format=format)
        self.columns.append(column)
        self.data[column] = [''] * self.row_count

    def delete_column(self, column):
        """
        Delete the specified column data and the layout
        :param column: column to be deleted
        :type column: str
        """
        if column in self.keys:
            index = self.columns.index(column)
            if self.layout:
                try:
                    self.layout.DeleteColumn(index)
                except Exception as e:
                    print(e)
            self.data.pop(column)
            self.columns.pop(index)

    def set_data_in_layout(self):
        """
        Retrieve data from self.data, convert to row_data format, add to layout
        """
        row_data = self.data_to_list_of_rows()

        for row in row_data:
            self.append_row(row, layout_only=True)

    def append_row(self, row, layout_only=False):
        """
        Add a row of data
        :param row: data ordered by self.columns
        :type row: list
        :param layout_only: If true, only add row to the GUI
        :type layout_only: bool
        """
        if not layout_only:
            self.append_row_to_data(row)
        if self.layout:
            index = self.layout.InsertItem(50000, str(row[0]))
            for i in range(len(row))[1:]:
                if isinstance(row[i], float) or isinstance(row[i], int) and str(row[i]) not in {'True', 'False'}:
                    value = "%0.2f" % row[i]
                else:
                    value = str(row[i])
                self.layout.SetItem(index, i, value)

    def append_row_to_data(self, row):
        """
        Add a row of data to self.data
        :param row: data ordered by self.columns
        :type row: list
        """
        if not self.data:
            columns = self.keys
            self.data = {columns[i]: [value] for i, value in enumerate(row)}
        else:
            for i, key in enumerate(self.keys):
                self.data[key].append(row[i])

    def edit_row_to_data(self, row, index):
        """
        Replace a row of data with a provided row and index, edits self.data only
        :param row: data ordered by self.columns
        :type row: list
        :param index: the row index
        :type index: int
        """
        for i, key in enumerate(self.keys):
            self.data[key][index] = row[i]

    def delete_row(self, index, layout_only=False):
        """
        Delete a specific row of data
        :param index: the row index
        :type index: int
        :param layout_only: If True, do not remove the row from self.data
        :type layout_only: bool
        """
        if index is not None:
            if not layout_only:
                for key in self.keys:
                    self.data[key].pop(index)
            if self.layout:
                self.layout.DeleteItem(index)

    def delete_all_rows(self, layout_only=False, force_delete_data=False):
        """
        Clear all data from self.data and the layout view
        :param layout_only: If True, do not remove the row from self.data
        :type layout_only: bool
        :param force_delete_data: If true, force deletion even if layout is not set
        """
        if self.layout:
            self.layout.DeleteAllItems()

        if self.layout or force_delete_data:
            if not layout_only:
                if self.data:
                    for key in self.keys:
                        self.data[key] = []

    def edit_row(self, row, index):
        """
        Replace a row of data with a provided row and index, update table view
        :param row: data ordered by self.columns
        :type row: list
        :param index: the row index
        :type index: int
        """
        self.edit_row_to_data(row, index)
        if self.layout:
            for i in range(len(row)):
                self.layout.SetItem(index, i, str(row[i]))

    def get_value(self, row_index, column_index):
        """
        Get a specific table value with a column name and row index
        :param row_index: retrieve value from row with this index
        :type row_index: int
        :param column_index: retrieve value from column with this index
        :type column_index: int
        :return: value corresponding to provided indices
        """
        return self.data[self.keys[column_index]][row_index]

    def get_row(self, row_index):
        """
        Get a row of data from self.data wtih the given row index
        :param row_index: retrieve all values from row with this index
        :type row_index: int
        :return: values for the specified row
        :rtype: list
        """
        return [self.data[key][row_index] for key in self.keys]

    def set_column_width(self, index, width):
        """
        Change the column width in the view
        :param index: index of column
        :type index: int
        :param width: the specified width
        :type width: int
        """
        self.layout.SetColumnWidth(index, width)

    def set_column_widths(self, auto=False):
        """Set all widths in layout based on self.widths"""
        if auto:
            for i in range(len(self.columns)):
                self.set_column_width(i, wx.LIST_AUTOSIZE_USEHEADER)
        else:
            if self.widths is not None:
                for i, width in enumerate(self.widths):
                    self.set_column_width(i, width)

    def clear(self):
        """Delete all data in self.data and clear the table view"""
        self.delete_all_rows()
        self.layout.DeleteAllColumns()

    def get_csv(self, extra_column_data=None):
        """
        This function will return a csv string of the data currently in this object
        :param extra_column_data: if there is additional data you'd like in the csv, pass the column data as a
        dictionary with:
            keys corresponding to column index
            values are dicts like {'title': str, 'data': list_of_data}
        :type extra_column_data: dict
        :return: csv string
        :rtype: str
        """

        data = deepcopy(self.data_for_csv)
        if extra_column_data:
            self.insert_columns_into_data_for_csv(data, extra_column_data)
        csv_data = []
        for row in data:
            csv_data.append(','.join(str(i) for i in row))

        return '\n'.join(csv_data)

    @property
    def data_for_csv(self):
        """
        Iterate through self.data to get a list of csv rows
        :return: csv data
        :rtype: list
        """
        data = [self.columns]
        for row_index in range(self.row_count):
            row = []
            for key in self.keys:
                raw_value = self.data[key][row_index]
                if isinstance(raw_value, float):
                    value = "%0.2f" % raw_value
                else:
                    value = str(raw_value).replace(',', ';')
                row.append(value)
            data.append(row)
        return data

    @staticmethod
    def insert_column_into_data_for_csv(data_for_csv, columns_dict_value, index):
        """
        Insert a column of data into the data returned from data_for_csv
        :param data_for_csv: rows of csv data
        :type data_for_csv: list
        :param columns_dict_value: title and data information. see extra_column_information in get_csv
        :type columns_dict_value: dict
        :param index: index of column to be inserted
        :type index: int
        """
        columns_dict_value['data'].insert(0, columns_dict_value['title'])
        for i, row in enumerate(data_for_csv):
            row.insert(index, str(columns_dict_value['data'][i]).replace(',', ';'))

    def insert_columns_into_data_for_csv(self, data_for_csv, columns_dict):
        """
        Insert columns_dict data into the provided data_for_csv
        :param data_for_csv: return from data_for_csv function
        :type data_for_csv: list
        :param columns_dict: extra columns data formatted as specified in get_csv
        :type columns_dict: dict
        """
        indices = list(columns_dict)
        indices.sort()
        for index in indices:
            self.insert_column_into_data_for_csv(data_for_csv, columns_dict[index], index)

    @property
    def selected_row_data(self):
        """
        :return: row data of the currently selected row in the GUI
        :rtype: list
        """
        return [self.get_row(index) for index in get_selected_listctrl_items(self.layout)]

    @property
    def selected_row_data_with_index(self):
        return [[index, self.get_row(index)] for index in get_selected_listctrl_items(self.layout)]

    def apply_selection_to_all(self, state):
        """
        Select or Deselect all rows in the layout
        :param state: True for selection, False for deselection
        :type state: bool
        """
        for i in range(self.row_count):
            self.layout.Select(i, on=state)

    @property
    def has_data(self):
        return bool(self.row_count)

    def sort_table(self, evt):

        if self.data:
            key = self.columns[evt.Column]  # get the column name from the column index (evt.Column)
            sort_indices = get_sorted_indices(self.data[key])  # handles str and float mixtures

            if self.sort_indices is None:
                self.sort_indices = list(range(len(self.data[key])))

            # reverse order if already sorted
            if sort_indices == list(range(len(sort_indices))):
                sort_indices = sort_indices[::-1]

            self.sort_indices = [self.sort_indices[i] for i in sort_indices]  # keep original order

            # reorder data and reinitialize table view
            self.data = {column: [self.data[column][i] for i in sort_indices] for column in self.columns}
            self.set_data(self.data, self.columns, self.formats)

    def get_data_in_original_order(self):
        if self.sort_indices is None:
            return self.data
        return {column: [self.data[column][i] for i in self.sort_indices] for column in self.columns}

    def get_unique_values(self, column):
        return sorted(set(self.data[column]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# models.database_editor.py
"""
Class for the Database Editor accessible from the main view
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics


import wx
from dvha.dialogs.database import ChangePatientIdentifierDialog, DeletePatientDialog, ReimportDialog, EditDatabaseDialog,\
    CalculationsDialog, DeleteAllData, RebuildDB, SQLErrorDialog
from dvha.db.sql_to_python import get_database_tree
from dvha.db.sql_connector import DVH_SQL, SQLError
from dvha.models.data_table import DataTable
from dvha.dialogs.export import save_data_to_file
from dvha.tools.utilities import set_msw_background_color, get_window_size, set_frame_icon
from dvha.models.roi_map import RemapROIFrame


class DatabaseEditorFrame(wx.Frame):
    """
    Various viewing and editing tools for the SQL database. This object is called on Database toolbar click.
    """
    def __init__(self, roi_map, options):
        """
        :param roi_map: roi_map object
        :type roi_map: DatabaseROIs
        """
        wx.Frame.__init__(self, None, title='Database Administrator')
        set_frame_icon(self)

        set_msw_background_color(self)  # If windows, change the background color

        self.roi_map = roi_map
        self.options = options
        self.db_tree = self.get_db_tree()

        self.SetSize(get_window_size(0.792, 0.781))

        self.window_db_editor = wx.SplitterWindow(self, wx.ID_ANY, style=wx.SP_3D)
        self.window_pane_db_tree = wx.ScrolledWindow(self.window_db_editor, wx.ID_ANY,
                                                     style=wx.BORDER_SUNKEN | wx.TAB_TRAVERSAL)
        self.tree_ctrl_db = wx.TreeCtrl(self.window_pane_db_tree, wx.ID_ANY, style=wx.TR_HAS_BUTTONS | wx.TR_MULTIPLE)
        self.window_pane_query = wx.Panel(self.window_db_editor, wx.ID_ANY, style=wx.BORDER_SUNKEN | wx.TAB_TRAVERSAL)
        self.text_ctrl_condition = wx.TextCtrl(self.window_pane_query, wx.ID_ANY, "")
        self.list_ctrl_query_results = wx.ListCtrl(self.window_pane_query, wx.ID_ANY,
                                                   style=wx.LC_HRULES | wx.LC_REPORT | wx.LC_VRULES)
        self.data_query_results = DataTable(self.list_ctrl_query_results, columns=['mrn', 'study_instance_uid'])
        self.combo_box_query_table = wx.ComboBox(self.window_pane_query, wx.ID_ANY, choices=list(self.db_tree),
                                                 style=wx.CB_DROPDOWN | wx.CB_READONLY)

        self.button = {'delete_all_data': wx.Button(self, wx.ID_ANY, "Delete All Data"),
                       'rebuild_db': wx.Button(self, wx.ID_ANY, "Rebuild Database"),
                       # 'calculations': wx.Button(self, wx.ID_ANY, "Calculations"),
                       'edit_db': wx.Button(self, wx.ID_ANY, "Edit Database"),
                       'reimport': wx.Button(self, wx.ID_ANY, "Reimport from DICOM"),
                       'delete_study': wx.Button(self, wx.ID_ANY, "Delete Study"),
                       'change_mrn_uid': wx.Button(self, wx.ID_ANY, "Change MRN/UID"),
                       'query': wx.Button(self.window_pane_query, wx.ID_ANY, "Query"),
                       'clear': wx.Button(self.window_pane_query, wx.ID_ANY, "Clear"),
                       'export_csv': wx.Button(self.window_pane_query, wx.ID_ANY, "Export"),
                       'remap_roi_names': wx.Button(self, wx.ID_ANY, "Remap ROI Names")}

        self.__set_properties()
        self.__do_layout()
        self.__do_bind()

        self.selected_columns = {table: {c: False for c in list(self.db_tree[table])} for table in list(self.db_tree)}
        self.selected_tables = {table: False for table in list(self.db_tree)}

        self.window_db_editor.SetSashPosition(250)
        self.tree_ctrl_db.Expand(self.db_tree_root)

        self.allow_tree_select_change = True

        self.Show()

    def __set_properties(self):
        self.SetTitle("Database Administrator")
        self.window_pane_db_tree.SetScrollRate(10, 10)
        self.window_db_editor.SetMinimumPaneSize(20)

        self.db_tree_root = self.tree_ctrl_db.AddRoot('DVH Analytics')
        self.table_nodes = {table: self.tree_ctrl_db.AppendItem(self.db_tree_root, table) for table in self.db_tree}
        self.column_nodes = {table: {} for table in self.db_tree}
        for table in self.db_tree:
            for column in self.db_tree[table]:
                self.column_nodes[table][column] = self.tree_ctrl_db.AppendItem(self.table_nodes[table], column)
        self.combo_box_query_table.SetValue('Plans')

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_query = wx.BoxSizer(wx.VERTICAL)
        sizer_query_table = wx.StaticBoxSizer(wx.StaticBox(self.window_pane_query, wx.ID_ANY, "Query Results"),
                                              wx.VERTICAL)
        sizer_condition_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_download_button = wx.BoxSizer(wx.VERTICAL)
        sizer_clear_button = wx.BoxSizer(wx.VERTICAL)
        sizer_query_button = wx.BoxSizer(wx.VERTICAL)
        sizer_condition = wx.BoxSizer(wx.VERTICAL)
        sizer_combo_box = wx.BoxSizer(wx.VERTICAL)
        sizer_db_tree = wx.BoxSizer(wx.HORIZONTAL)
        sizer_dialog_buttons = wx.BoxSizer(wx.HORIZONTAL)

        # sizer_dialog_buttons.Add(self.button['calculations'], 0, wx.ALL, 5)
        sizer_dialog_buttons.Add(self.button['edit_db'], 0, wx.ALL, 5)
        sizer_dialog_buttons.Add(self.button['reimport'], 0, wx.ALL, 5)
        sizer_dialog_buttons.Add(self.button['delete_study'], 0, wx.ALL, 5)
        sizer_dialog_buttons.Add(self.button['change_mrn_uid'], 0, wx.ALL, 5)
        sizer_dialog_buttons.Add(self.button['rebuild_db'], 0, wx.ALL, 5)
        sizer_dialog_buttons.Add(self.button['delete_all_data'], 0, wx.ALL, 5)
        sizer_dialog_buttons.Add(self.button['remap_roi_names'], 0, wx.ALL, 5)
        sizer_wrapper.Add(sizer_dialog_buttons, 0, wx.ALL, 5)

        sizer_db_tree.Add(self.tree_ctrl_db, 1, wx.EXPAND, 0)
        self.window_pane_db_tree.SetSizer(sizer_db_tree)

        label_table = wx.StaticText(self.window_pane_query, wx.ID_ANY, "Table:")
        sizer_combo_box.Add(label_table, 0, wx.BOTTOM | wx.TOP, 5)
        sizer_combo_box.Add(self.combo_box_query_table, 0, 0, 0)
        sizer_condition_buttons.Add(sizer_combo_box, 0, wx.ALL | wx.EXPAND, 5)

        label_condition = wx.StaticText(self.window_pane_query, wx.ID_ANY, "Condition:")
        sizer_condition.Add(label_condition, 0, wx.BOTTOM | wx.TOP, 5)
        sizer_condition.Add(self.text_ctrl_condition, 0, wx.ALL | wx.EXPAND, 0)
        sizer_condition_buttons.Add(sizer_condition, 1, wx.ALL | wx.EXPAND, 5)

        label_spacer_1 = wx.StaticText(self.window_pane_query, wx.ID_ANY, "")
        sizer_query_button.Add(label_spacer_1, 0, wx.BOTTOM, 5)
        sizer_query_button.Add(self.button['query'], 0, wx.ALL, 5)
        sizer_condition_buttons.Add(sizer_query_button, 0, wx.ALL, 5)

        label_spacer_2 = wx.StaticText(self.window_pane_query, wx.ID_ANY, "")
        sizer_download_button.Add(label_spacer_2, 0, wx.BOTTOM, 5)
        sizer_download_button.Add(self.button['export_csv'], 0, wx.TOP | wx.BOTTOM, 5)
        sizer_condition_buttons.Add(sizer_download_button, 0, wx.ALL, 5)

        label_spacer_3 = wx.StaticText(self.window_pane_query, wx.ID_ANY, "")
        sizer_clear_button.Add(label_spacer_3, 0, wx.BOTTOM, 5)
        sizer_clear_button.Add(self.button['clear'], 0, wx.ALL, 5)
        sizer_condition_buttons.Add(sizer_clear_button, 0, wx.ALL, 5)

        sizer_query.Add(sizer_condition_buttons, 0, wx.BOTTOM | wx.EXPAND | wx.LEFT | wx.RIGHT, 5)

        sizer_query_table.Add(self.list_ctrl_query_results, 1, wx.EXPAND, 0)
        sizer_query.Add(sizer_query_table, 1, wx.ALL | wx.EXPAND, 5)

        self.window_pane_query.SetSizer(sizer_query)
        self.window_db_editor.SplitVertically(self.window_pane_db_tree, self.window_pane_query)
        sizer_wrapper.Add(self.window_db_editor, 1, wx.EXPAND, 0)
        self.SetSizer(sizer_wrapper)
        self.Layout()
        self.Center()

    def __do_bind(self):
        # self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnTreeAdd, self.tree_ctrl_db, id=self.tree_ctrl_db.GetId())

        # All buttons are bound to a function based on their key prepended with 'on_'
        # For example, query button calls on_query when clicked
        for key, button in self.button.items():
            self.Bind(wx.EVT_BUTTON, getattr(self, 'on_' + key), id=button.GetId())

        self.Bind(wx.EVT_LIST_COL_CLICK, self.sort_query_results, self.list_ctrl_query_results)

    def on_tree_add(self, evt):
        self.update_selected_tree_items()

    # def UnselectOtherTables(self, selected_table):
    #     self.allow_tree_select_change = False
    #     for table, node in self.table_nodes.items():
    #         if table != selected_table:
    #             self.tree_ctrl_db.UnselectItem(node)
    #     self.allow_tree_select_change = True

    def on_query(self, evt):
        self.update_selected_tree_items()
        table = self.combo_box_query_table.GetValue()
        columns = [c for c, sel in self.selected_columns[table].items()
                   if sel and c not in {'mrn', 'study_instance_uid'}]
        columns.sort()

        if not columns:
            columns = [c for c in self.db_tree[table] if c not in {'mrn', 'study_instance_uid'}]

        columns.insert(0, 'study_instance_uid')
        columns.insert(0, 'mrn')

        condition = self.text_ctrl_condition.GetValue()

        with DVH_SQL() as cnx:
            try:
                data = cnx.query(table, ','.join(columns), condition, bokeh_cds=True)
                self.data_query_results.set_data(data, columns)
            except SQLError as e:
                SQLErrorDialog(self, e)
                self.data_query_results.clear()

    def on_clear(self, evt):
        self.data_query_results.clear()

    @staticmethod
    def get_db_tree():
        tree = get_database_tree()
        for table in list(tree):
            tree[table] = [column for column in tree[table] if 'string' not in column]
        return tree

    def update_selected_tree_items(self):
        for table, table_item in self.table_nodes.items():
            self.selected_tables[table] = self.tree_ctrl_db.IsSelected(table_item)
            for column, column_item in self.column_nodes[table].items():
                self.selected_columns[table][column] = self.tree_ctrl_db.IsSelected(column_item)

    def on_change_mrn_uid(self, evt):
        self.change_or_delete_dlg(ChangePatientIdentifierDialog)

    def on_delete_study(self, evt):
        self.change_or_delete_dlg(DeletePatientDialog)

    def change_or_delete_dlg(self, class_type):
        selected_data = self.data_query_results.selected_row_data
        if selected_data:
            class_type(mrn=selected_data[0][0], study_instance_uid=selected_data[0][1])
        else:
            class_type()

    def on_reimport(self, evt):
        selected_data = self.data_query_results.selected_row_data
        if selected_data:
            ReimportDialog(self.roi_map, self.options, mrn=selected_data[0][0], study_instance_uid=selected_data[0][1])
        else:
            ReimportDialog(self.roi_map, self.options)

    @staticmethod
    def on_edit_db(evt):
        EditDatabaseDialog()

    @staticmethod
    def on_calculations(evt):
        CalculationsDialog()

    def on_rebuild_db(self, evt):
        RebuildDB(self, self.roi_map, self.options)

    def on_delete_all_data(self, evt):
        DeleteAllData(self, self.options)

    def on_export_csv(self, evt):
        save_data_to_file(self, "Export Data Table to CSV", self.data_query_results.get_csv())

    def on_remap_roi_names(self, evt):
        RemapROIFrame(self.roi_map, remap_all=True)

    def sort_query_results(self, evt):
        self.data_query_results.sort_table(evt)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# models.time_series.py
"""
Class for viewing and editing the roi map, and updating the database with changes
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

import wx
from dateutil.parser import parse as date_parser
from dvha.db import sql_columns
from dvha.dialogs.export import save_data_to_file
from dvha.models.plot import PlotTimeSeries


class TimeSeriesFrame:
    """
    Object to be passed into notebook panel for the Time Series tab
    """
    def __init__(self, parent, group_data, options):
        """
        :param parent:  notebook panel in main view
        :type parent: Panel
        :param group_data: dvh data object
        :type group_data: DVH
        :param options: user options containing visual preferences
        :type options: Options
        """
        self.parent = parent
        self.options = options
        self.stats_data = {grp: data['stats_data'] for grp, data in group_data.items()}

        self.combo_box_y_axis = wx.ComboBox(self.parent, wx.ID_ANY, style=wx.CB_DROPDOWN | wx.CB_READONLY)
        self.text_input_bin_size = wx.TextCtrl(self.parent, wx.ID_ANY, "10", style=wx.TE_PROCESS_ENTER)
        self.text_input_lookback_distance = wx.TextCtrl(self.parent, wx.ID_ANY, "1", style=wx.TE_PROCESS_ENTER)
        self.text_inputs_percentile = wx.TextCtrl(self.parent, wx.ID_ANY, "90", style=wx.TE_PROCESS_ENTER)
        self.button_update_plot = wx.Button(self.parent, wx.ID_ANY, "Update Plot")
        self.button_export_csv = wx.Button(self.parent, wx.ID_ANY, "Export")

        self.parent.Bind(wx.EVT_COMBOBOX, self.combo_box_y_axis_ticker, id=self.combo_box_y_axis.GetId())
        self.parent.Bind(wx.EVT_TEXT_ENTER, self.update_plot_ticker, id=self.text_input_bin_size.GetId())
        self.parent.Bind(wx.EVT_TEXT_ENTER, self.update_plot_ticker, id=self.text_input_lookback_distance.GetId())
        self.parent.Bind(wx.EVT_TEXT_ENTER, self.update_plot_ticker, id=self.text_inputs_percentile.GetId())

        self.__set_properties()
        self.__do_layout()

        self.parent.Bind(wx.EVT_BUTTON, self.update_plot_ticker, id=self.button_update_plot.GetId())
        self.parent.Bind(wx.EVT_BUTTON, self.export_csv, id=self.button_export_csv.GetId())

        self.disable_buttons()

        self.layout_save_attr = ['combo_box_y_axis', 'text_input_lookback_distance',
                                 'text_inputs_percentile', 'text_input_bin_size']

    def __set_properties(self):
        self.combo_box_y_axis.AppendItems(self.choices)
        self.combo_box_y_axis.SetLabelText('ROI Max Dose')

    @property
    def choices(self):
        if self.stats_data[1]:
            choices = self.stats_data[1].trending_variables
            if self.stats_data[2]:
                choices.extend(self.stats_data[2].trending_variables)
                choices = list(set(choices))
        else:
            choices = list(sql_columns.numerical)
        choices.sort()
        return choices

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_plot = wx.BoxSizer(wx.HORIZONTAL)
        sizer_widgets = wx.StaticBoxSizer(wx.StaticBox(self.parent, wx.ID_ANY, ""), wx.HORIZONTAL)
        sizer_percentile = wx.BoxSizer(wx.VERTICAL)
        sizer_lookback_distance = wx.BoxSizer(wx.VERTICAL)
        sizer_histogram_bins = wx.BoxSizer(wx.VERTICAL)
        sizer_y_axis = wx.BoxSizer(wx.VERTICAL)

        label_y_axis = wx.StaticText(self.parent, wx.ID_ANY, "Y Axis:")
        sizer_y_axis.Add(label_y_axis, 0, wx.LEFT, 5)
        sizer_y_axis.Add(self.combo_box_y_axis, 0, wx.ALL | wx.EXPAND, 5)
        sizer_widgets.Add(sizer_y_axis, 1, wx.EXPAND, 0)

        label_histogram_bins = wx.StaticText(self.parent, wx.ID_ANY, "Histogram Bins:")
        sizer_histogram_bins.Add(label_histogram_bins, 0, wx.LEFT, 5)
        sizer_histogram_bins.Add(self.text_input_bin_size, 0, wx.ALL | wx.EXPAND, 5)

        label_lookback_distance = wx.StaticText(self.parent, wx.ID_ANY, "Lookback Distance:")
        sizer_lookback_distance.Add(label_lookback_distance, 0, wx.LEFT, 5)
        sizer_lookback_distance.Add(self.text_input_lookback_distance, 0, wx.ALL | wx.EXPAND, 5)
        sizer_widgets.Add(sizer_lookback_distance, 1, wx.EXPAND, 0)

        label_percentile = wx.StaticText(self.parent, wx.ID_ANY, "Percentile:")
        sizer_percentile.Add(label_percentile, 0, wx.LEFT, 5)
        sizer_percentile.Add(self.text_inputs_percentile, 0, wx.ALL | wx.EXPAND, 5)
        sizer_widgets.Add(sizer_percentile, 1, wx.EXPAND, 0)
        sizer_widgets.Add(sizer_histogram_bins, 1, wx.EXPAND, 0)
        sizer_widgets.Add(self.button_update_plot, 0, wx.ALL | wx.EXPAND, 5)
        sizer_widgets.Add(self.button_export_csv, 0, wx.ALL | wx.EXPAND, 5)
        sizer_wrapper.Add(sizer_widgets, 0, wx.BOTTOM | wx.EXPAND, 5)

        self.plot = PlotTimeSeries(self.parent, self.options)
        sizer_plot.Add(self.plot.layout, 1, wx.EXPAND, 0)
        sizer_wrapper.Add(sizer_plot, 1, wx.EXPAND, 0)

        self.layout = sizer_wrapper

    def combo_box_y_axis_ticker(self, evt):
        self.update_plot()

    def update_plot(self):
        data = self.get_plot_data()
        self.plot.update_plot(data)

    def get_plot_data(self, y_axis_selection=None):
        if y_axis_selection is None:
            y_axis_selection = self.combo_box_y_axis.GetValue()
        data = {}
        for grp, stats_data in self.stats_data.items():
            if stats_data:
                y_data = stats_data.data[y_axis_selection]['values']
                x_data = stats_data.sim_study_dates

                sort_index = sorted(range(len(x_data)), key=lambda k: x_data[k])
                x_values_sorted, y_values_sorted, mrn_sorted, uid_sorted = [], [], [], []

                for s in range(len(x_data)):
                    try:
                        x = date_parser(x_data[sort_index[s]])
                    except Exception:
                        continue
                    x_values_sorted.append(x)
                    y_values_sorted.append(y_data[sort_index[s]])
                    mrn_sorted.append(stats_data.mrns[sort_index[s]])
                    uid_sorted.append(stats_data.uids[sort_index[s]])

                data[grp] = {'x': x_values_sorted,
                             'y': y_values_sorted,
                             'mrn': mrn_sorted,
                             'uid': uid_sorted,
                             'y_axis_label': stats_data.get_axis_title(y_axis_selection),
                             'avg_len': self.avg_len,
                             'percentile': self.percentile,
                             'bin_size': self.hist_bins,
                             'group': [grp] * len(x_values_sorted)}
            else:
                data[grp] = {'x': [], 'y': [], 'mrn': [], 'uid': [], 'y_axis_label': [], 'avg_len': [],
                             'percentile': [], 'bin_size': [], 'group': []}
        return data

    @property
    def hist_bins(self):
        try:
            hist_bins = int(self.text_input_bin_size.GetValue())
        except ValueError:
            self.text_input_bin_size.SetValue('10')
            hist_bins = 10
        return hist_bins

    @property
    def avg_len(self):
        try:
            avg_len = int(self.text_input_lookback_distance.GetValue())
        except ValueError:
            self.text_input_lookback_distance.SetValue('1')
            avg_len = 1
        return avg_len

    @property
    def percentile(self):
        try:
            percentile = float(self.text_inputs_percentile.GetValue())
        except ValueError:
            self.text_inputs_percentile.SetValue('90')
            percentile = 90.
        return percentile

    def update_data(self, group_data):
        self.stats_data = {grp: data['stats_data'] for grp, data in group_data.items()}
        self.initialize_y_axis_options()
        self.update_plot()

    def clear_data(self):
        self.plot.clear_plot()
        self.initialize_y_axis_options()
        self.combo_box_y_axis.SetLabelText('ROI Max Dose')

    def enable_buttons(self):
        self.button_update_plot.Enable()

    def disable_buttons(self):
        self.button_update_plot.Disable()

    def enable_initial_buttons(self):
        self.button_update_plot.Enable()

    def update_plot_ticker(self, evt):
        self.update_plot()

    def update_y_axis_options(self):
        current_choice = self.combo_box_y_axis.GetValue()
        self.combo_box_y_axis.SetItems(self.choices)
        if current_choice not in self.choices:
            current_choice = 'ROI Max Dose'
        self.combo_box_y_axis.SetValue(current_choice)

    def initialize_y_axis_options(self):
        self.combo_box_y_axis.SetItems(self.choices)
        self.combo_box_y_axis.SetValue('ROI Max Dose')

    def get_save_data(self):
        return {attr: getattr(self, attr).GetValue() for attr in self.layout_save_attr}

    def load_save_data(self, save_data):
        for attr in list(save_data):
            getattr(self, attr).SetValue(save_data[attr])
        self.update_y_axis_options()

    def get_csv(self, selection=None):
        """
        :param selection: variables to be included
        :type selection: list
        :return: csv data
        :rtype: str
        """

        # get_csv may be called from Time Series tab or DVHA app menu or tool bar
        # if selection is None, get_csv was called from Time Series tab, only export data in plot
        if selection is None:
            return self.plot.get_csv()

        # if selection is not None, export being called from DVHA app menu or tool bar

        csv = []
        for grp in [1, 2]:
            if self.stats_data[grp]:
                uids = self.stats_data[grp].uids
                mrns = self.stats_data[grp].mrns
                dates = self.stats_data[grp].sim_study_dates

                # Collect y-data (as in y-axis data from time series), organize into dict for printing to rows
                y_data = {}
                for y_axis in selection:
                    data = self.get_plot_data(y_axis_selection=y_axis)[grp]
                    column = []
                    for uid in uids:
                        if uid in data['uid']:
                            index = data['uid'].index(uid)
                            column.append(data['y'][index])
                        else:
                            column.append('None')
                    y_data[y_axis] = column

                if grp == 2:
                    csv.insert(0, 'Group 1')
                    csv.append('\nGroup 2')

                # Collect data into a list of row data
                csv.append('MRN,Study Instance UID,Date,%s' % ','.join(selection))
                for i, uid in enumerate(uids):
                    row = [mrns[i], uid, str(dates[i])]
                    for y_axis in selection:
                        row.append(str(y_data[y_axis][i]))
                    csv.append(','.join(row))

        return '\n'.join(csv)

    def export_csv(self, evt):
        save_data_to_file(self.parent, "Export Time Series data to CSV", self.plot.get_csv())

    @property
    def has_data(self):
        return self.button_export_csv.IsEnabled()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# models.endpoint.py
"""
Class for the Endpoint frame in the main view
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

import wx
from copy import deepcopy
from dvha.models.data_table import DataTable
from dvha.dialogs.main import AddEndpointDialog, DelEndpointDialog
from dvha.dialogs.export import save_data_to_file
from dvha.tools.stats import sync_variables_in_stats_data_objects
from dvha.tools.utilities import get_window_size


class EndpointFrame:
    """
    Object to be passed into notebook panel for the Endpoint tab
    """
    def __init__(self, parent, group_data, time_series, regression, control_chart):
        """
        :param parent:  notebook panel in main view
        :type parent: Panel
        :param group_data: dvh, table_data, and stats_data
        :type group_data: dict
        :param time_series: Time Series object in notebook
        :type time_series: TimeSeriesFrame
        :param regression: Regression frame object in notebook
        :type regression: RegressionFrame
        :param control_chart: Control Chart frame object in notebook
        :type control_chart: ControlChartFrame
        """

        self.parent = parent
        self.group_data = group_data
        self.time_series = time_series
        self.regression = regression
        self.control_chart = control_chart
        self.group_data = group_data
        self.initial_columns = ['MRN', 'Tx Site', 'ROI Name', 'Volume (cc)']
        self.widths = [150, 150, 250, 100]

        self.button = {'add': wx.Button(self.parent, wx.ID_ANY, "Add Endpoint"),
                       'del': wx.Button(self.parent, wx.ID_ANY, "Delete Endpoint"),
                       'exp': wx.Button(self.parent, wx.ID_ANY, 'Export')}

        self.table = {key: wx.ListCtrl(self.parent, wx.ID_ANY,
                                       style=wx.BORDER_SUNKEN | wx.LC_HRULES | wx.LC_REPORT | wx.LC_VRULES)
                      for key in [1, 2]}
        for table in self.table.values():
            table.SetMinSize(get_window_size(0.623, 0.28))
        self.data_table = {key: DataTable(self.table[key]) for key in [1, 2]}

        self.endpoint_defs = DataTable(None, columns=['label', 'output_type', 'input_type',
                                                      'input_value', 'units_in', 'units_out'])

        for key in [1, 2]:
            if self.group_data[key]['dvh']:
                self.group_data[key]['dvh'].endpoints['data'] = self.data_table[key].data
                self.group_data[key]['dvh'].endpoints['defs'] = self.endpoint_defs.data

        self.__do_bind()
        self.__set_properties()
        self.__do_layout()

        self.disable_buttons()

    def __do_bind(self):
        self.parent.Bind(wx.EVT_BUTTON, self.add_ep_button_click, id=self.button['add'].GetId())
        self.parent.Bind(wx.EVT_BUTTON, self.del_ep_button_click, id=self.button['del'].GetId())
        self.parent.Bind(wx.EVT_BUTTON, self.on_export_csv, id=self.button['exp'].GetId())

    def __set_properties(self):
        for table in self.table.values():
            for i, column in enumerate(self.initial_columns):
                table.AppendColumn(column, format=wx.LIST_FORMAT_LEFT, width=self.widths[i])

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        vbox = wx.BoxSizer(wx.VERTICAL)
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        for key in list(self.button):
            hbox.Add(self.button[key], 0, wx.ALL, 5)
        vbox.Add(hbox, 0, wx.ALL | wx.EXPAND, 5)
        for grp in [1, 2]:
            vbox.Add(wx.StaticText(self.parent, wx.ID_ANY, "Query Group %s:" % grp), 0, wx.EXPAND | wx.BOTTOM, 5)
            vbox.Add(self.table[grp], 1, wx.EXPAND, 0)
            vbox.Add((10, 10), 0, 0, 0)
        sizer_wrapper.Add(vbox, 1, wx.ALL | wx.EXPAND, 20)
        self.layout = sizer_wrapper

    def calculate_endpoints(self):

        columns = {key: [c for c in self.initial_columns] for key in [1, 2]}
        if self.data_table[1].data:
            current_labels = [key for key in list(self.data_table[1].data) if key not in columns]
        else:
            current_labels = []

        eps = {grp: {'MRN': group_data['dvh'].mrn,
                     'Tx Site': group_data['dvh'].get_plan_values('tx_site'),
                     'ROI Name': group_data['dvh'].roi_name,
                     'Volume (cc)': group_data['dvh'].volume}
               for grp, group_data in self.group_data.items() if group_data['dvh']}

        ep_defs = self.endpoint_defs.data
        for group, ep in eps.items():
            if ep_defs:
                for i, ep_name in enumerate(ep_defs['label']):

                    if ep_name not in columns[group]:
                        columns[group].append(ep_name)

                        if ep_name in current_labels:
                            ep[ep_name] = deepcopy(self.data_table[group].data[ep_name])

                        else:
                            endpoint_input = ep_defs['input_type'][i]
                            endpoint_output = ep_defs['output_type'][i]

                            x = float(ep_defs['input_value'][i])
                            if endpoint_input == 'relative':
                                x /= 100.

                            dvh = self.group_data[group]['dvh']
                            if 'V' in ep_name:
                                ep[ep_name] = dvh.get_volume_of_dose(x, volume_scale=endpoint_output,
                                                                     dose_scale=endpoint_input)
                            else:
                                ep[ep_name] = dvh.get_dose_to_volume(x, dose_scale=endpoint_output,
                                                                     volume_scale=endpoint_input)

        for group, ep in eps.items():
            self.data_table[group].set_data(ep, columns[group])
            self.data_table[group].set_column_width(0, 150)
            self.data_table[group].set_column_width(1, 150)
            self.data_table[group].set_column_width(2, 200)

    def add_ep_button_click(self, evt):
        # TODO: Track down duplicate table refreshes/calculations
        dlg = AddEndpointDialog()
        res = dlg.ShowModal()
        if res == wx.ID_OK and dlg.is_endpoint_valid:
            self.endpoint_defs.append_row(dlg.endpoint_row)
            self.calculate_endpoints()
            self.enable_buttons()
            self.update_endpoints_in_dvh()
        dlg.Destroy()
        self.update_endpoints_and_radbio_in_group_data()
        self.regression.update_combo_box_choices()
        self.control_chart.update_combo_box_y_choices()
        self.time_series.update_y_axis_options()

    def del_ep_button_click(self, evt):
        dlg = DelEndpointDialog(self.data_table[1].columns)
        res = dlg.ShowModal()
        if res == wx.ID_OK:
            for value in dlg.selected_values:
                for data_table in self.data_table.values():
                    if data_table.columns and value in data_table.columns:
                        data_table.delete_column(value)
                endpoint_def_row = self.endpoint_defs.data['label'].index(value)
                self.update_endpoints_in_dvh()
                self.endpoint_defs.delete_row(endpoint_def_row)
                self.update_endpoints_and_radbio_in_group_data()

            self.time_series.update_y_axis_options()
        dlg.Destroy()

        for group in self.group_data.values():
            if group['stats_data']:
                group['stats_data'].update_endpoints_and_radbio()
        self.regression.update_combo_box_choices()
        self.control_chart.update_combo_box_y_choices()

        for data_table in self.data_table.values():
            if data_table.column_count == 3:
                self.button['del'].Disable()
                self.button['exp'].Disable()

    def update_dvh(self, group_data):
        self.group_data = group_data
        self.update_endpoints_in_dvh()

    def update_endpoints_and_radbio_in_group_data(self):
        for grp, data in self.group_data.items():
            if data['stats_data']:
                data['stats_data'].update_endpoints_and_radbio()
                if grp == 2:
                    sync_variables_in_stats_data_objects(self.group_data[1]['stats_data'],
                                                         self.group_data[2]['stats_data'])

    def update_endpoints_in_dvh(self):
        for group in [1, 2]:
            if self.group_data[group]['dvh']:
                self.group_data[group]['dvh'].endpoints['data'] = self.data_table[group].data
                self.group_data[group]['dvh'].endpoints['defs'] = self.endpoint_defs.data

    def clear_data(self):
        for data_table in self.data_table.values():
            data_table.delete_all_rows()
        self.endpoint_defs.delete_all_rows(force_delete_data=True)  # no attached layout, force delete

        for data_table in self.data_table.values():
            if data_table.data:
                for column in list(data_table.data):
                    if column not in self.initial_columns:
                        data_table.delete_column(column)

    def enable_buttons(self):
        for key in list(self.button):
            self.button[key].Enable()

    def disable_buttons(self):
        for key in list(self.button):
            self.button[key].Disable()

    def enable_initial_buttons(self):
        self.button['add'].Enable()

    def get_csv(self, selection=None):
        uid = {1: {'title': 'Study Instance UID',
                   'data': self.group_data[1]['dvh'].uid}}
        csv = self.data_table[1].get_csv(extra_column_data=uid)
        if self.group_data[2]['dvh']:
            uid = {1: {'title': 'Study Instance UID',
                       'data': self.group_data[2]['dvh'].uid}}
            csv = 'Group 1\n%s\n\nGroup 2\n%s' % (csv, self.data_table[2].get_csv(extra_column_data=uid))

        return csv

    def on_export_csv(self, evt):
        save_data_to_file(self.parent, "Export Endpoints to CSV", self.get_csv())

    def get_save_data(self):
        return {'data_table_1': self.data_table[1].get_save_data(),
                'data_table_2': self.data_table[2].get_save_data(),
                'endpoint_defs': self.endpoint_defs.get_save_data()}

    def load_save_data(self, save_data):
        self.data_table[1].load_save_data(save_data['data_table_1'], ignore_layout=True)
        self.data_table[2].load_save_data(save_data['data_table_2'], ignore_layout=True)
        self.endpoint_defs.load_save_data(save_data['endpoint_defs'], ignore_layout=True)
        self.calculate_endpoints()

    @property
    def has_data(self):
        if self.endpoint_defs.data and self.endpoint_defs.data['label']:
            return True
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# dialogs.export.py
"""
GUI tools to export text data to a file
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

import wx
import wx.adv
from functools import partial
import matplotlib.colors as plot_colors
from dvha.models.data_table import DataTable
from dvha.paths import DATA_DIR
from dvha.tools.errors import ErrorDialog
from dvha.tools.utilities import get_selected_listctrl_items, save_object_to_file,\
    set_msw_background_color, set_frame_icon


def save_data_to_file(frame, title, data, wildcard="CSV files (*.csv)|*.csv", data_type='string', initial_dir=DATA_DIR):
    """
    from https://wxpython.org/Phoenix/docs/html/wx.FileDialog.html
    :param frame: GUI parent
    :param title: title for the file dialog window
    :type title: str
    :param data: text data or pickle-able object to be written
    :param wildcard: restrict visible files and intended file extension
    :type wildcard: str
    :param data_type: either 'string' or 'pickle'
    :type data_type: str
    :param initial_dir: start the FileDialog at this directory
    :type initial_dir: str
    """

    with wx.FileDialog(frame, title, defaultDir=initial_dir,  wildcard=wildcard,
                       style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT) as fileDialog:

        if fileDialog.ShowModal() == wx.ID_CANCEL:
            return

        pathname = fileDialog.GetPath()

        if data_type == 'string':
            try:
                with open(pathname, 'w', encoding="utf-8") as file:
                    file.write(data)
            except IOError:
                wx.LogError("Cannot save current data in file '%s'." % pathname)

        elif data_type == 'pickle':
            save_object_to_file(data, pathname)

        elif data_type == 'function':
            data(pathname)


class ExportCSVDialog(wx.Dialog):
    """
    Allow user to select available data for export to CSV into one file.
    This class leverages the get_csv functions built-into each of the data types / tabs
    """
    def __init__(self, app):
        """
        :param app: easier to pass main frame pointer than several links to each data type
        :type app: DVHAMainFrame
        """
        wx.Dialog.__init__(self, None)

        self.app = app

        # Each of these objects shoudl have a has_data property, if False, those UI elements in this class will
        # be disabled (i.e., don't allow user to export empty tables
        self.enabled = {'DVHs': self.app.group_data[1]['dvh'].has_data,
                        'DVHs Summary': self.app.group_data[1]['dvh'].has_data,
                        'Endpoints': self.app.endpoint.has_data,
                        'Radbio': self.app.radbio.has_data,
                        'Charting Variables': self.app.time_series.has_data}

        checkbox_keys = ['DVHs', 'DVHs Summary', 'Endpoints', 'Radbio', 'Charting Variables']
        self.checkbox = {key: wx.CheckBox(self, wx.ID_ANY, key) for key in checkbox_keys}

        # set to a dictionary because previous versions had a tree with Regression
        self.list_ctrl = {'Charting Variables': wx.ListCtrl(self, wx.ID_ANY,
                                                            style=wx.LC_HRULES | wx.LC_REPORT | wx.LC_VRULES)}

        time_series_column = "Variables"
        time_series_variables = self.app.time_series.combo_box_y_axis.GetItems()
        time_series_data = {time_series_column: time_series_variables}
        self.data_table_time_series = DataTable(self.list_ctrl['Charting Variables'],
                                                columns=[time_series_column], widths=[400])
        self.data_table_time_series.set_data(time_series_data, [time_series_column])

        # set to a dictionary because previous versions had a table with Regression
        self.button_select_data = {'Charting Variables': {'Select': wx.Button(self, wx.ID_ANY, "Select All"),
                                                          'Deselect': wx.Button(self, wx.ID_ANY, "Deselect All")}}

        self.button_save = wx.Button(self, wx.ID_OK, "Save")
        self.button_cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")
        self.button_select_all = wx.Button(self, wx.ID_ANY, 'Select All')
        self.button_deselect_all = wx.Button(self, wx.ID_ANY, 'Deselect All')

        self.__set_properties()
        self.__do_bind()
        self.__do_layout()

        self.run()

    def __set_properties(self):
        self.SetTitle("Export Data to CSV")
        self.button_select_all.SetToolTip('Only data objects with data will be enabled.')
        self.validate_ui_objects()

    def __do_bind(self):
        self.Bind(wx.EVT_BUTTON, self.on_select_all, id=self.button_select_all.GetId())
        self.Bind(wx.EVT_BUTTON, self.on_deselect_all, id=self.button_deselect_all.GetId())
        self.Bind(wx.EVT_BUTTON, self.on_time_series_select_all,
                  id=self.button_select_data['Charting Variables']['Select'].GetId())
        self.Bind(wx.EVT_BUTTON, self.on_time_series_deselect_all,
                  id=self.button_select_data['Charting Variables']['Deselect'].GetId())

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_main = wx.BoxSizer(wx.VERTICAL)
        sizer_main_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_data = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, "Data Selection"), wx.VERTICAL)
        sizer_time_series = wx.BoxSizer(wx.VERTICAL)
        sizer_time_series_listctrl = wx.BoxSizer(wx.HORIZONTAL)
        sizer_time_series_checkboxes = wx.BoxSizer(wx.HORIZONTAL)
        sizer_time_series_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_radbio = wx.BoxSizer(wx.VERTICAL)
        sizer_endpoints = wx.BoxSizer(wx.VERTICAL)
        sizer_dvhs = wx.BoxSizer(wx.VERTICAL)
        sizer_dvhs_checkboxes = wx.BoxSizer(wx.HORIZONTAL)

        keys = ['DVHs', 'Endpoints', 'Radbio']
        static_line = {key: wx.StaticLine(self, wx.ID_ANY) for key in keys}

        sizer_dvhs_checkboxes.Add(self.checkbox['DVHs'], 1, wx.ALL | wx.EXPAND, 5)
        sizer_dvhs_checkboxes.Add(self.checkbox['DVHs Summary'], 1, wx.ALL | wx.EXPAND, 5)
        sizer_dvhs.Add(sizer_dvhs_checkboxes, 1, wx.EXPAND, 0)
        sizer_dvhs.Add(static_line['DVHs'], 0, wx.EXPAND | wx.TOP, 5)
        sizer_data.Add(sizer_dvhs, 0, wx.ALL | wx.EXPAND, 5)

        sizer_endpoints.Add(self.checkbox['Endpoints'], 0, wx.ALL, 5)
        sizer_endpoints.Add(static_line['Endpoints'], 0, wx.EXPAND | wx.TOP, 5)
        sizer_data.Add(sizer_endpoints, 0, wx.ALL | wx.EXPAND, 5)

        sizer_radbio.Add(self.checkbox['Radbio'], 0, wx.ALL, 5)
        sizer_radbio.Add(static_line['Radbio'], 0, wx.EXPAND | wx.TOP, 5)
        sizer_data.Add(sizer_radbio, 0, wx.ALL | wx.EXPAND, 5)

        sizer_time_series_checkboxes.Add(self.checkbox['Charting Variables'], 1, wx.EXPAND, 0)
        sizer_time_series_buttons.Add(self.button_select_data['Charting Variables']['Select'],
                                      0, wx.ALL | wx.EXPAND, 5)
        sizer_time_series_buttons.Add(self.button_select_data['Charting Variables']['Deselect'],
                                      0, wx.ALL | wx.EXPAND, 5)
        sizer_time_series_checkboxes.Add(sizer_time_series_buttons, 1, wx.EXPAND, 0)
        sizer_time_series.Add(sizer_time_series_checkboxes, 0, wx.ALL | wx.EXPAND, 5)
        sizer_time_series_listctrl.Add((20, 20), 0, 0, 0)
        sizer_time_series_listctrl.Add(self.list_ctrl['Charting Variables'], 1, wx.ALL | wx.EXPAND, 5)
        sizer_time_series.Add(sizer_time_series_listctrl, 0, wx.ALL | wx.EXPAND, 5)
        sizer_data.Add(sizer_time_series, 0, wx.ALL | wx.EXPAND, 5)

        sizer_main.Add(sizer_data, 0, wx.ALL | wx.EXPAND, 5)

        sizer_main_buttons.Add(self.button_select_all, 0, wx.ALL, 5)
        sizer_main_buttons.Add(self.button_deselect_all, 0, wx.ALL, 5)

        sizer_main_buttons.Add(self.button_save, 0, wx.ALL, 5)
        sizer_main_buttons.Add(self.button_cancel, 0, wx.ALL, 5)
        sizer_main.Add(sizer_main_buttons, 0, wx.ALIGN_RIGHT | wx.ALL, 5)

        sizer_wrapper.Add(sizer_main, 1, wx.EXPAND, 0)

        self.SetSizer(sizer_wrapper)
        self.Layout()
        self.Fit()
        self.Center()

    def set_checkbox_values(self, value):
        for checkbox in self.checkbox.values():
            checkbox.SetValue(value)

    def on_select_all(self, evt):
        self.set_checkbox_values(True)
        self.validate_ui_objects(allow_enable=False)

    def on_deselect_all(self, evt):
        self.set_checkbox_values(False)
        self.validate_ui_objects(allow_enable=False)

    def on_time_series_select_all(self, evt):
        self.data_table_time_series.apply_selection_to_all(1)

    def on_time_series_deselect_all(self, evt):
        self.data_table_time_series.apply_selection_to_all(0)

    def validate_ui_objects(self, allow_enable=True):
        tables = {'Charting Variables': self.data_table_time_series}
        for key, data_table in tables.items():
            state = data_table.has_data
            if not state or (state and allow_enable):
                self.list_ctrl[key].Enable(state)
                self.button_select_data[key]['Select'].Enable(state)
                self.button_select_data[key]['Deselect'].Enable(state)

        for key, value in self.enabled.items():
            if not value or (value and allow_enable):
                self.checkbox[key].SetValue(value)
                self.checkbox[key].Enable(value)

    def run(self):
        res = self.ShowModal()
        if res == wx.ID_OK:
            save_data_to_file(self, 'Export CSV Data', self.csv)
        self.Destroy()

    def is_checked(self, key):
        return self.checkbox[key].GetValue()

    def on_dvh_check(self, evt):
        if not self.is_checked('DVHs'):
            self.checkbox['DVHs Summary'].SetValue(False)

    @property
    def csv(self):
        csv_data = []

        csv_key = ['DVHs', 'Endpoints', 'Radbio', 'Charting Variables']
        csv_obj = [None, self.app.endpoint, self.app.radbio, self.app.time_series, self.app.control_chart]
        for i, key in enumerate(csv_key):
            if self.is_checked(key) or (key == 'DVHs' and self.is_checked('DVHs Summary')):
                csv_data.append('%s\n' % key)
                if key == 'DVHs':  # DVHs has a summary and plot data for export
                    csv_data.append(self.app.plot.get_csv(include_summary=self.is_checked('DVHs Summary'),
                                                          include_dvhs=self.is_checked('DVHs')))
                else:
                    if key == 'Charting Variables':
                        selection_indices = get_selected_listctrl_items(self.list_ctrl['Charting Variables'])
                        y_choices = self.app.time_series.combo_box_y_axis.GetItems()
                        selection = [y for i, y in enumerate(y_choices) if i in selection_indices]
                    else:
                        selection = None
                    csv_data.append(csv_obj[i].get_csv(selection=selection))
                csv_data.append('\n\n')

        return '\n'.join(csv_data)


class ExportFigure(wx.Frame):
    def __init__(self, parent):
        wx.Frame.__init__(self, parent)

        self.parent = parent
        self.options = parent.options
        self.plots = {'DVHs': parent.plot,
                      'Time Series': parent.time_series.plot,
                      'Regression': parent.regression.plot,
                      'Control Chart': parent.control_chart.plot}

        keys_tc = ['y_range_start', 'y_range_end', 'x_range_start', 'plot_height', 'plot_width']
        self.text_ctrl = {key: wx.TextCtrl(self, wx.ID_ANY, str(self.options.save_fig_param[key])) for key in keys_tc}
        self.label = {key: wx.StaticText(self, wx.ID_ANY, key.replace('_', ' ').title() + ':') for key in keys_tc}

        self.keys_cb = ['background_fill_color', 'border_fill_color', 'plot', 'format']
        self.combo_box = {key: wx.ComboBox(self, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN | wx.TE_READONLY)
                          for key in self.keys_cb}
        for key in self.keys_cb:
            self.label[key] = wx.StaticText(self, wx.ID_ANY, key.replace('_', ' ').title() + ':')

        self.button_keys = ['Export', 'Dismiss']
        self.button = {'Export': wx.Button(self, wx.ID_ANY, 'Export'),
                       'Dismiss': wx.Button(self, wx.ID_CANCEL, 'Dismiss')}

        self.keys = keys_tc + self.keys_cb
        self.fig_attr_keys = keys_tc + ['background_fill_color', 'border_fill_color']

        self.__set_properties()
        self.__do_bind()
        self.__do_layout()

        set_msw_background_color(self)
        set_frame_icon(self)

    def __set_properties(self):
        self.SetTitle("Export Figure")

        self.combo_box['plot'].SetItems(sorted(list(self.plots)))
        self.combo_box['plot'].SetValue('DVHs')

        self.combo_box['format'].SetItems(['HTML', 'SVG'])
        self.combo_box['format'].SetValue('SVG')

        color_options = ['none'] + list(plot_colors.cnames)
        for key in ['background_fill_color', 'border_fill_color']:
            self.combo_box[key].SetItems(color_options)
            self.combo_box[key].SetValue(self.options.save_fig_param[key])

    def __do_bind(self):
        self.Bind(wx.EVT_BUTTON, self.on_export, id=self.button['Export'].GetId())
        self.Bind(wx.EVT_BUTTON, self.on_dismiss, id=wx.ID_CANCEL)
        self.Bind(wx.EVT_CLOSE, self.on_close)

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_main = wx.BoxSizer(wx.VERTICAL)
        sizer_input = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, ""), wx.VERTICAL)
        sizer_buttons = wx.BoxSizer(wx.HORIZONTAL)

        for key in self.keys:
            input_obj = self.combo_box[key] if key in self.keys_cb else self.text_ctrl[key]
            sizer_input.Add(self.label[key], 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, 5)
            sizer_input.Add(input_obj, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.BOTTOM, 5)
        sizer_main.Add(sizer_input, 0, wx.EXPAND | wx.ALL, 5)

        for key in self.button_keys:
            sizer_buttons.Add(self.button[key], 0, wx.ALIGN_RIGHT | wx.ALL, 5)
        sizer_main.Add(sizer_buttons, 0, wx.EXPAND | wx.ALL, 5)

        sizer_wrapper.Add(sizer_main, 0, wx.EXPAND | wx.ALL, 5)

        self.SetSizer(sizer_wrapper)
        self.Fit()
        self.Layout()
        self.Center()

    @property
    def plot(self):
        return self.plots[self.combo_box['plot'].GetValue()]

    @property
    def export_title(self):
        return "Save %s plot as .%s" % (self.combo_box['plot'].GetValue(), self.format.lower())

    @property
    def wildcard(self):
        ext = self.combo_box['format'].GetValue()
        return "%s files (*.%s)|*.%s" % (ext.upper(), ext.lower(), ext.lower())

    @property
    def format(self):
        return self.combo_box['format'].GetValue()

    @property
    def save_plot_function(self):
        return partial(self.plot.save_figure, self.format.lower(), self.fig_attr_dict)

    def on_export(self, *evt):
        try:
            save_data_to_file(self, self.export_title, self.save_plot_function, initial_dir="",
                              data_type='function', wildcard=self.wildcard)
        except Exception as e:
            ErrorDialog(self, str(e), "Save Error")

    def get_text_ctrl_float(self, key):
        value = self.text_ctrl[key].GetValue()
        try:
            return float(value)
        except ValueError:
            return None

    def get_text_ctrl_int(self, key):
        value = self.text_ctrl[key].GetValue()
        try:
            return int(float(value))
        except ValueError:
            return None

    def get_combo_box(self, key):
        return self.combo_box[key].GetValue()

    @property
    def y_range_start(self):
        return self.get_text_ctrl_float('y_range_start')

    @property
    def x_range_start(self):
        return self.get_text_ctrl_float('x_range_start')

    @property
    def y_range_end(self):
        return self.get_text_ctrl_float('y_range_end')

    @property
    def plot_height(self):
        return self.get_text_ctrl_int('plot_height')

    @property
    def plot_width(self):
        return self.get_text_ctrl_int('plot_width')

    @property
    def background_fill_color(self):
        return self.get_combo_box('background_fill_color')

    @property
    def border_fill_color(self):
        return self.get_combo_box('border_fill_color')

    @property
    def fig_attr_dict(self):
        return {key: getattr(self, key) for key in self.fig_attr_keys}

    def on_dismiss(self, *evt):
        wx.CallAfter(self.on_close)

    def on_close(self, *evt):
        self.save_options()
        self.parent.export_figure = None
        self.Destroy()

    def save_options(self):
        self.options.save_fig_param = self.fig_attr_dict
        self.options.save()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# tools.roi_geometry.py
"""
Tools for geometric calculations

"sets of points" objects
    dictionaries using str(z) as keys
        where z is the slice or z in DICOM coordinates
        each item is a list of points representing a polygon, each point is a 3-item list [x, y, z]

"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

from scipy.spatial.distance import cdist
import numpy as np
from math import ceil
from dvha.tools.roi_formatter import points_to_shapely_polygon, dicompyler_roi_to_sets_of_points,\
    get_shapely_from_sets_of_points


def union(rois):
    """
    Calculate the geometric union of the provided rois
    :param rois: rois formatted as "sets of points" dictionaries
    :type rois: list
    :return: a "sets of points" dictionary representing the union of the rois
    :rtype: dict
    """

    new_roi = {}

    all_z_values = []
    for roi in rois:
        for z in list(roi):
            if z not in all_z_values:
                all_z_values.append(z)

    for z in all_z_values:

        if z not in list(new_roi):
            new_roi[z] = []

        # Convert to shapely objects
        current_slice = None
        for roi in rois:
            # Make sure current roi has at least 3 points in z plane
            if z in list(roi) and len(roi[z][0]) > 2:
                if not current_slice:
                    current_slice = points_to_shapely_polygon(roi[z])
                else:
                    current_slice = current_slice.union(points_to_shapely_polygon(roi[z]))

        if current_slice:
            if current_slice.type != 'MultiPolygon':
                current_slice = [current_slice]

            for polygon in current_slice:
                xy = polygon.exterior.xy
                x_coord = xy[0]
                y_coord = xy[1]
                points = []
                for i in range(len(x_coord)):
                    points.append([x_coord[i], y_coord[i], round(float(z), 2)])
                new_roi[z].append(points)

                if hasattr(polygon, 'interiors'):
                    for interior in polygon.interiors:
                        xy = interior.coords.xy
                        x_coord = xy[0]
                        y_coord = xy[1]
                        points = []
                        for i in range(len(x_coord)):
                            points.append([x_coord[i], y_coord[i], round(float(z), 2)])
                        new_roi[z].append(points)
        else:
            print('WARNING: no contour found for slice %s' % z)

    return new_roi


def min_distances_to_target(oar_coordinates, target_coordinates):
    """
    Calculate all OAR-point-to-Target-point euclidean distances
    :param oar_coordinates: numpy arrays of 3D points defining the surface of the OAR
    :type oar_coordinates: list
    :param target_coordinates: numpy arrays of 3D points defining the surface of the PTV
    :type target_coordinates: list
    :return: min_distances: all minimum distances (cm) of OAR-point-to-Target-point pairs
    :rtype: list
    """
    # TODO: This very computationally expensive, needs a sampling method prior to calling cdist
    min_distances = []
    all_distances = cdist(oar_coordinates, target_coordinates, 'euclidean')
    for oar_point in all_distances:
        min_distances.append(float(np.min(oar_point)/10.))

    return min_distances


def cross_section(roi):
    """
    Calculate the cross section of a given roi
    :param roi: a "sets of points" formatted dictionary
    :type roi: dict
    :return: max and median cross-sectional area of all slices in cm^2
    :rtype: dict
    """
    areas = []

    for z in list(roi):
        shapely_roi = points_to_shapely_polygon(roi[z])
        if shapely_roi and shapely_roi.area > 0:
            slice_centroid = shapely_roi.centroid
            polygon_count = len(slice_centroid.xy[0])
            for i in range(polygon_count):
                if polygon_count > 1:
                    areas.append(shapely_roi[i].area)
                else:
                    areas.append(shapely_roi.area)

    areas = np.array(areas)

    area = {'max': float(np.max(areas) / 100.),
            'median': float(np.median(areas) / 100.)}

    return area


def surface_area(coord, coord_type='dicompyler'):
    """
    Calculate the surface of a given roi
    :param coord: dicompyler structure coordinates from GetStructureCoordinates() or a sets_of_points dictionary
    :param coord_type: either 'dicompyler' or 'sets_of_points'
    :return: surface_area in cm^2
    :rtype: float
    """
    # TODO: This surface area method needs validation, but likely needs to be corrected

    if coord_type == "sets_of_points":
        sets_of_points = coord
    else:
        sets_of_points = dicompyler_roi_to_sets_of_points(coord)

    shapely_roi = get_shapely_from_sets_of_points(sets_of_points)

    slice_count = len(shapely_roi['z'])

    area = 0.
    polygon = shapely_roi['polygon']
    z = shapely_roi['z']
    thickness = min(shapely_roi['thickness'])

    for i in range(slice_count):
        for j in [-1, 1]:  # -1 for bottom area and 1 for top area
            # ensure bottom of first slice and top of last slice are fully added
            # if prev/next slice is not adjacent, assume non-contiguous ROI
            if (i == 0 and j == -1) or (i == slice_count-1 and j == 1) or abs(z[i] - z[i+j]) > 2*thickness:
                area += polygon[i].area
            else:
                area += polygon[i].difference(polygon[i+j]).area

        area += polygon[i].length * thickness

    return round(area/100, 3)


def overlap_volume(oar, tv):
    """
    Calculate the overlap volume of two rois
    :param oar: organ-at-risk as a "sets of points" formatted dictionary
    :type oar: dict
    :param tv: treatment volume as a "sets of points" formatted dictionary
    :type tv: dict
    :rtype: float
    """

    intersection_volume = 0.
    all_z_values = [round(float(z), 2) for z in list(tv)]
    all_z_values = np.sort(all_z_values)
    thicknesses = np.abs(np.diff(all_z_values))
    if len(thicknesses) > 0:
        thicknesses = np.append(thicknesses, np.min(thicknesses))
        all_z_values = all_z_values.tolist()

        for z in list(tv):
            # z in coord will not necessarily go in order of z, convert z to float to lookup thickness
            # also used to check for top and bottom slices, to add area of those contours

            if z in list(oar):
                thickness = thicknesses[all_z_values.index(round(float(z), 2))]
                shapely_tv = points_to_shapely_polygon(tv[z])
                shapely_oar = points_to_shapely_polygon(oar[z])
                if shapely_oar and shapely_tv:
                    intersection_volume += shapely_tv.intersection(shapely_oar).area * thickness

        return round(intersection_volume / 1000., 2)
    return 0.


def volume(roi):
    """
    :param roi: a "sets of points" formatted dictionary
    :return: volume in cm^3 of roi
    :rtype: float
    """

    # oar and ptv are lists using str(z) as keys
    # each item is an ordered list of points representing a polygon
    # polygon n is inside polygon n-1, then the current accumulated polygon is
    #    polygon n subtracted from the accumulated polygon up to and including polygon n-1
    #    Same method DICOM uses to handle rings and islands

    vol = 0.
    all_z_values = [round(float(z), 2) for z in list(roi)]
    all_z_values = np.sort(all_z_values)
    thicknesses = np.abs(np.diff(all_z_values))
    thicknesses = np.append(thicknesses, np.min(thicknesses))
    all_z_values = all_z_values.tolist()

    for z in list(roi):
        # z in coord will not necessarily go in order of z, convert z to float to lookup thickness
        # also used to check for top and bottom slices, to add area of those contours

        thickness = thicknesses[all_z_values.index(round(float(z), 2))]
        shapely_roi = points_to_shapely_polygon(roi[z])
        if shapely_roi:
            vol += shapely_roi.area * thickness

    return round(vol / 1000., 2)


def centroid(roi):
    """
    :param roi: a "sets of points" formatted dictionary
    :return: centroid or the roi in x, y, z dicom coordinates (mm)
    :rtype: list
    """
    centroids = {'x': [], 'y': [], 'z': [], 'area': []}

    for z in list(roi):
        shapely_roi = points_to_shapely_polygon(roi[z])
        if shapely_roi and shapely_roi.area > 0:
            slice_centroid = shapely_roi.centroid
            polygon_count = len(slice_centroid.xy[0])
            for i in range(polygon_count):
                centroids['x'].append(slice_centroid.xy[0][i])
                centroids['y'].append(slice_centroid.xy[1][i])
                centroids['z'].append(float(z))
                if polygon_count > 1:
                    centroids['area'].append(shapely_roi[i].area)
                else:
                    centroids['area'].append(shapely_roi.area)

    x = np.array(centroids['x'])
    y = np.array(centroids['y'])
    z = np.array(centroids['z'])
    w = np.array(centroids['area'])
    w_sum = np.sum(w)

    volumetric_centroid = [float(np.sum(x * w) / w_sum),
                           float(np.sum(y * w) / w_sum),
                           float(np.sum(z * w) / w_sum)]

    return volumetric_centroid


def spread(roi):
    """
    :param roi: a "sets of points" formatted dictionary
    :return: x, y, z dimensions of a rectangular prism encompassing roi
    :rtype: list
    """
    all_points = {'x': [], 'y': [], 'z': []}

    for z in list(roi):
        for polygon in roi[z]:
            for point in polygon:
                all_points['x'].append(point[0])
                all_points['y'].append(point[1])
                all_points['z'].append(point[2])

    all_points = {'x': np.array(all_points['x']),
                  'y': np.array(all_points['y']),
                  'z': np.array(all_points['z'])}

    if len(all_points['x'] > 1):
        data = [abs(float(np.max(all_points[dim]) - np.min(all_points[dim]))) for dim in ['x', 'y', 'z']]
    else:
        data = [0, 0, 0]

    return data


def dth(min_distances):
    """
    :param min_distances: the output from min_distances_to_target
    :return: histogram of distances in 0.1mm bin widths
    :rtype: numpy.array
    """

    bin_count = int(ceil(np.max(min_distances) * 10.))
    data, bins = np.histogram(min_distances, bins=bin_count)

    return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# setup.py
"""
A setuptools setup file for DVH Analytics
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

from setuptools import setup, find_packages
from dvha._version import __version__


with open('requirements.txt', 'r') as doc:
    requires = [line.strip() for line in doc]

with open('README.md', 'r') as doc:
    long_description = doc.read()


setup(
    name='dvha',
    include_package_data=True,
    python_requires='>3.5',
    packages=find_packages(),
    version=__version__,
    description='Create a database of DVHs, GUI with wxPython, plots with Bokeh',
    author='Dan Cutright',
    author_email='dan.cutright@gmail.com',
    url='https://github.com/cutright/DVH-Analytics-Desktop',
    download_url='https://github.com/cutright/DVH-Analytics-Desktop/archive/master.zip',
    license="BSD License",
    keywords=['dvh', 'radiation therapy', 'research', 'dicom', 'dicom-rt', 'bokeh', 'analytics', 'wxpython'],
    classifiers=[],
    install_requires=requires,
    entry_points={'console_scripts': ['dvha = dvha.main:start']},
    long_description=long_description,
    long_description_content_type="text/markdown"
)
